#$Id$#

from os.path import basename
from json import dumps
from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.InvoicesParser import InvoicesParser
from .Api import Api

base_url = Api().base_url + 'invoices/'
parser = InvoicesParser()
zoho_http_client = ZohoHttpClient()

class InvoicesApi:
    """Invoice Api class is used to:
    
    1.List all invoices with pagination.
    2.Get the details of an invoice.
    3.Create an invoice.
    4.Update an existing invoice.
    5.Delete an existing invoice.
    6.Mark a draft invoice as sent.
    7.Mark an invoice status as void.
    8.Mark a voided invoice as draft.
    9.Email an invoice to the customer.
    10.Send invoices to your customer by email.
    11.Get the email content of an email.
    12.Remind the customer about an unpaid invoice by email.
    13.Remind the customer abount unpaid invoices by email.
    14.Get the mail content of the payment reminder.
    15.Export maximum of 25 invoices as pdf.
    16.Export invoices as pdf and print them.
    17.Disable automated payment reminders for an invoice.
    18.Enable automated payment reminders for an invoice.
    19.Write off the invoice balance amount of an invoice.
    20.Cancel the write off amount of an invoice.
    21.Update the billing address for an existing invoice.
    22.Update the shipping address for an existing invoice.
    23.Get all invoice pdf templates.
    24.Update the pdf template associated with the invoice.
    25.Get the list of payments made for an invoice.
    26.Get the list of credits applied for an invoice.
    27.Apply the customer credits to an invoice.
    28.Delete a payment made to an invoice.
    29.Delete a particular credit applied to an invoice.
    30.Return a file attached to an invoice.
    31.Attach a file to an invoice.
    32.Send attachment while emailing the invoice.
    33.Delete the file attached to the invoice.
    34.Delete an expense receipt attached to an invoice.
    35.Get the complete history and comments of an invoice.
    36.Add a comment for an invoice.
    37.Update an existing comment of an invoice.
    38.Delete an invoice comment.

    """
    def __init__(self, authtoken, organization_id):
        """Initialize Invoice api using user's authtoken and organization 
            id.

        Args:
            authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details = {
            'authtoken':authtoken, 
            'organization_id':organization_id
            }

    def get_invoices(self, parameter=None):
        """Get list of all invoices with pagination.

        Args:
            parameter(dict, optional): Filter with which the list has to be 
                displayed.
   
        Returns:
            instance: Invoice list instance.
 
        """
        response = zoho_http_client.get(base_url, self.details, parameter)
        return parser.get_list(response)   
  
    def get(self, invoice_id, print_pdf=None, accept=None):
        """Get details of an invoice.
 
        Args: 
            invoice_id(str): Inovice id.
            print_pdf(bool): True to print invoice as pdf else False.
            accept(str): Format in which the invoice details has to be 
                downloaded. Default value is json. Allowed value is json, 
                pdf and html.

        Returns:
            instance or file: If accept is None invoice object is returned 
                else File containing invoice details is returned.
        
        """            
        url = base_url + invoice_id
        query = {}
        if print_pdf is not None and accept is not None:
            query = {
                'print': print_pdf, 
                'accept': accept
                }
            resp = zoho_http_client.getfile(url, self.details, query)
            return resp
        elif print_pdf is not None:
             query = {'print': print_pdf} 
             if print_pdf is True:
                 query.update({'accept':'pdf'})
                 resp = zoho_http_client.getfile(url, self.details, query)
                 return  resp
             else:
                 response = zoho_http_client.get(url, self.details, query)
                 return parser.get(response) 
        elif accept is not None:
            query = {
                'accept': accept
                }
            resp = zoho_http_client.getfile(url, self.details, query)
            return resp 
        else:
            response = zoho_http_client.get(url, self.details)
            return parser.get(response)
   
    def create(self, invoice, send=None, ignore_auto_number_generation=None):
        """Creat an invoice.

        Args:
            invoice(instance): Invoice object.
            send(bool, optional): To send invoice to the cocntact persons 
                associated with the invoice. Allowed values are true and false.
            ignore_auto_number_generation(bool, optional): Ignore auto invoice 
                number generation for the invoice. This mandates the invoice 
                number. Allowed values are true and false.

        Returns:
            instance: Invoice object.

        """
        json_object = dumps(invoice.to_json())
        data = { 
            'JSONString': json_object
            }
        query = {}
        if send is not None and ignore_auto_number_generation is not None:
            query = {
                'send':send, 
                'ignore_auto_number_generation': ignore_auto_number_generation
                }         
        elif send is not None or ignore_auto_number_generation is not None:
            query = {
                'send': send
                } if send is not None else {
                    'ignore_auto_number_generation': \
                    ignore_auto_number_generation
                    }
        else: 
            query = None
        response = zoho_http_client.post(base_url, self.details, data, query)
        return parser.get(response)  

    def update(self, invoice_id, invoice, ignore_auto_number_generation=None):
        """Update an existing invoice.

        Args:
            invoie_id(str): Invoice id.
            invoice(instance): Invoice object.
            ignore_auto_number_generation(bool, optional): Ignore auto invoice 
                number generation for the invoice. This mandates the invoice 
                number. Allowed values are true and false.

        Returns:
            instance: Invoice object.

        """
        url = base_url + invoice_id
        json_object = dumps(invoice.to_json())
        data = { 
            'JSONString': json_object
            }
        if ignore_auto_number_generation is not None:
            query = {
                'ignore_auto_number_generation': ignore_auto_number_generation
                }
        else:
            query = None
        response = zoho_http_client.put(url, self.details, data, query)
        return parser.get(response) 
      
    def delete(self, invoice_id):
        """Delete an existing invoice.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            str: Success message('The invoice has been deleted.').

        """
        url = base_url + invoice_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 
   
    def mark_an_invoice_as_sent(self, invoice_id):
        """Mark an invoice as sent.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            str: Success message('Inovice status has been changed to sent.').
        
        """
        url = base_url + invoice_id + '/status/sent'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response)  

    def void_an_invoice(self, invoice_id): 
        """Mark an invoice as void.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            str: Success message('Invoice status has been changed to void.').
        
        """
        url = base_url + invoice_id + '/status/void'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response)   

    def mark_as_draft(self, invoice_id):
        """Mark a voided invoice as draft.

        Args: 
            invoice_id(str): Invoice id.

        Returns:
            str: Success message('Status of invoice changed form void to  
                    draft.').
 
        """
        url = base_url + invoice_id + '/status/draft/'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response)    

    def email_an_invoice(self, invoice_id, email, attachment=None, \
        send_customer_statement=None, send_attachment=None):
        """Email an invoice to the customer.

        Args:
            invoice_id(str): Invoice id.
            email(instance): Email object.
            attachment(list of dict, optional): List of dictionary objects 
                containing details of files to be attached.
            send_customer_statement(bool, optional): True to send customer 
                statement pdf with email else False.
            send_attachment(bool, optional): True to send the attachment with 
                mail else False.

        Returns:
            str: Success message('Your invoice has been sent.').

        """
        url = base_url + invoice_id + '/email'
        json_object = dumps(email.to_json())
        data = { 
            'JSONString': json_object
            }
        if attachment is not None and send_customer_statement is not None and \
            send_attachment is not None:
            query = {
                'send_customer_statement': send_customer_statement, 
                'send_attachment': send_attachment
                }
            file_list = []
            for value in attachment:
                attachments = {
                    'attachments': {
                        'filename': basename(value), 
                        'content':open(value).read()
                        }
                }
                file_list.append(attachments)
        elif attachment is not None and send_customer_statement is not None:
            query = {
                'send_customer_statement':send_customer_statement, 
                }
            file_list = []
            for value in attachment:
                attachments = {
                    'attachments': {
                        'filename': basename(value),
                        'content': open(value).read()
                        } 
                    }
                file_list.append(attachments)
        elif attachment is not None and send_attachment is not None:
            query = {
                'send_attachment':send_attachment
                }
            file_list = []
            for value in attachment:
                attachments = {
                    'attachments': {
                        'filename':basename(value), 
                        'content':open(value).read()
                        }
                    }
                file_list.append(attachments)
        elif send_customer_statement is not None and send_attachment is not None:
            query = {
                'send_customer_statement':send_customer_statement, 
                'send_attachment':send_attachment
                }
            file_list = None
        elif attachment is not None:
            file_list = []
            for value in attachment:
                attachments = {
                    'attachments': {
                        'filename': basename(value), 
                        'content':open(value).read()
                        }
                    }
                file_list.append(attachments)
            query = None
        elif send_customer_statement is not None:
            query = {
                'send_customer_statement':send_customer_statement, 
                }
            file_list = None
        elif send_attachment is not None:
            query = {
                'send_attachment':send_attachment
                }
            file_list = None
        else:
            query = None
            file_list = None
        response = zoho_http_client.post(url, self.details, data, query, file_list)
        return parser.get_message(response) 

    def email_invoices(self, contact_id, invoice_ids, email=None, \
        snail_mail=None):
        """Send invoice to customers by email.

        Args:
            contact_id(list of str): List of Contact ids.
            invoice_ids(str): Comma separated Invoice ids which are to 
                be emailed.
            email(bool, optional): True to send via email.
            snail_mail(bool, optional): True to send via snail mail.

        Returns:
            str: Success message('Mission accomplished! We've sent all 
                the invoices.').

        """
        query = {
            'invoice_ids': invoice_ids
            }
        url = base_url + 'email'
        data = {}
        data['contacts'] = []
        for value in contact_id:
            contacts = {
                'contact_id':value, 
                'email':True, 
                'snail_mail': False
                }
            if (email is not None) and (snail_mail is not None):
                contacts = {
                    'contact_id':value, 
                    'email':email, 
                    'snail_mail':snail_mail
                    }
            data['contacts'].append(contacts)
        fields = {
            'JSONString': dumps(data)
            }
        print(data)
        response = zoho_http_client.post(url, self.details, fields, query)
        return parser.get_message(response) 

    def get_email_content(self, invoice_id, email_template_id=None):
        """Get the email content of an invoice.
 
        Args:
            invoice_id(str): Invoice id.
            email_template_id(str, optional): Email template id. If None 
                 default template id will be inputted.

        Returns:
            instance: Email object.

        """
        url = base_url + invoice_id + '/email'
        if email_template_id is not None:
            query = {
                'email_template_id':email_template_id
                }
        else:
            query = None
        response = zoho_http_client.get(url, self.details, query)
        return parser.get_content(response) 

    def remind_customer(self, invoice_id, email, attachment=None, \
        send_customer_statement=None):      
        """Remind customers abount unpaid invoices.
      
        Args:
            invoice_id(str): Invoice id.
            email(instance): Email object.
            attachment(list of dict, optional): List of dictionary objects 
                containing details of files to be attached.
            send_customer_statement(bool, optional): True to send customer 
                statement along with email else False.
  
        Returns:
            str: Success message('Your payment reminder has been sent.').
 
        """
        url = base_url + invoice_id + '/paymentreminder'
        json_object = dumps(email.to_json())
        data = {
            'JSONString': json_object
            }
        if send_customer_statement is not None and attachment is not None:
           query = {
               'send_customer_statement':send_customer_statement
               }
           file_list = []
           for value in attachment:
               attachments = {
                   'attachments': {
                       'filename': basename(value), 
                       'content':open(value).read()
                       }
                   }
               file_list.append(attachments)
        elif send_customer_statement is not None:
            query = {
                'send_customer_statement':send_customer_statement
                }
            file_list = None
        elif attachment is not None:
            file_list = []
            for value in attachment:
                attachments = {
                    'attachments': {
                        'filename': basename(value), 
                        'content':open(value).read()
                        }
                    }
                file_list.append(attachments)
            query = None
        else:
            query = None
            file_list = None
        response = zoho_http_client.post(url, self.details, data, query, file_list)
        return parser.get_message(response)  

    def bulk_invoice_reminder(self, invoice_id):
        """Remind customers about unpaid invoices.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            str: Success message('Success! All reminders have been sent.').

        """
        url = base_url + 'paymentreminder'
        invoice_ids = {
            'invoice_ids': invoice_id
            }
        response = zoho_http_client.post(url, self.details, '', invoice_ids)
        return parser.get_message(response) 
 
    def get_payment_reminder_mail_content(self, invoice_id):
        """Get the mail content of the payment reminder.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            instance: Email object.

        """
        url = base_url + invoice_id + '/paymentreminder'
        response = zoho_http_client.get(url, self.details)
        return parser.payment_reminder_mail_content(response) 
 
    def bulk_export_invoices(self, invoice_id):
        """Export maximum of 25 invoices as pdf.
 
        Args:
            invoice_id(str): Comma separated invoice ids which are to be 
                exported as pdfs.
 
        Returns:
            file: Pdf file containing invoice details.
     
        """
        url = base_url + 'pdf'
        query = {
            'invoice_ids': invoice_id
            }
        response = zoho_http_client.getfile(url, self.details, query)
        return response 
   
    def bulk_print_invoices(self, invoice_id):
        """Export invoices as pdf and print them.
 
        Args:
            invoice_id(str): Invoice id.

        Returns:
            file: File that has to be printed.

        """
        url = base_url + 'pdf'
        invoice_ids = {
            'invoice_ids': invoice_id
            }
        response = zoho_http_client.getfile(url, self.details, invoice_ids)
        return response 
    
    def disable_payment_reminder(self, invoice_id):
        """Disable payment reminer.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            str: Success message('Reminders stopped.').

        """
        url = base_url + invoice_id + '/paymentreminder/disable'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 

    def enable_payment_reminder(self, invoice_id):
        """Enable payment reminder.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            str: Success message('Reminders enabled.').

        """
        url = base_url + invoice_id + '/paymentreminder/enable'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response)        
      
    def write_off_invoice(self, invoice_id):
        """Write off the invoice balance amount of an invoice.

        Args:
            invoice_id(str): Invoice id.
  
        Returns:
            str: Success message('Invoice has been written off.').

        """
        url = base_url + invoice_id + '/writeoff'  
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 
 
    def cancel_write_off(self, invoice_id):
        """Cancel the write off amount of an invoice.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            str: Success message('The writeoff done for this invoice 
                has been cancelled.').

        """
        url = base_url + invoice_id + '/writeoff/cancel'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 

    def update_billing_address(self, invoice_id, address, \
        is_update_customer=None):
        """Update billing address for the invoice.

        Args:
            invoice_id(str): Invoice id.
            address(instance): Address object.
            is_update_customer(bool, optional): True to update the address
                for all draft, unpaid invoice and future invoices.

        Returns:
            instance: Address object.

        """
        url = base_url + invoice_id + '/address/billing'
        json_object = dumps(address.to_json())
        data = {
            'JSONString': json_object
            }
        if is_update_customer is not None:
            query = {
                'is_update_customer': is_update_customer
                }
        else:
            query = None
        response = zoho_http_client.put(url, self.details, data, query)
        return parser.get_billing_address(response) 

    def update_shipping_address(self, invoice_id, address, \
        is_update_customer=None):
        """Update shipping address for the invoice.

        Args:
            invoice_id(str): Invoice id.
            address(instance): Address object.
            is_update_customer(bool, optional): True to update the address 
                for all draft, unpaid invoice and future invoices.

        Returns:
            instance: Address object.

        """
        url = base_url + invoice_id + '/address/shipping'
        json_object = dumps(address.to_json())
        data = {
            'JSONString': json_object
            }
        if is_update_customer is not None:
            query = {
                'is_update_customer': is_update_customer
                }
        else:
            query = None
        response = zoho_http_client.put(url, self.details, data, query)
        return parser.get_shipping_address(response)  

    def list_invoice_templates(self):
        """Get all invoice pdf templates.

        Returns: 
            instance: Invoice template list object.

        """
        url = base_url + 'templates'
        response = zoho_http_client.get(url, self.details)
        return parser.invoice_template_list(response) 

    def update_invoice_template(self, invoice_id, template_id):
        """Update the pdf template associated with the invoice.

        Args:
            invoice_id(str): Invoice id.
            template_id(str): Tempalte id.
 
        Returns:
            str: Success message('Invoice information has been updated.').
        
        """
        url = base_url + invoice_id + '/templates/' + template_id
        response = zoho_http_client.put(url, self.details, '')
        return parser.get_message(response) 

## Payments and Credits--------------------------------------------------------

    def list_invoice_payments(self, invoice_id):
        """List the payments made for an invoice.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            list of payments instance: List of Invoice payments list object.

        """
        url = base_url + invoice_id + '/payments'
        response = zoho_http_client.get(url, self.details)
        return parser.payments_list(response) 

    def list_credits_applied(self, invoice_id):
        """Get the list of credits applied for an invoice.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            list of credits instance: List of credits object.

        """
        url = base_url + invoice_id + '/creditsapplied'
        response = zoho_http_client.get(url, self.details)
        return parser.credits_list(response) 

    def apply_credits(self, invoice_id, payments_and_credits):
        """Apply the customer credits either from credit notes or 
            excess customer payments to an invoice.
 
        Args:
            invoice_id(str): Invoice id.
            payments_and_credits(instance): Payments object.

        Returns:
            instance: Payments and credits object.
 
        """
        url = base_url + invoice_id + '/credits'
        data = {}
        invoice_payments = []
        apply_credits = []
        for value in payments_and_credits.invoice_payments:
            payments = {}
            payments['payment_id'] = value.get_payment_id()
            payments['amount_applied'] = value.get_amount_applied()
            invoice_payments.append(payments)
        for value in payments_and_credits.apply_creditnotes:
            credits = {}
            credits['creditnote_id'] = value.get_creditnote_id()
            credits['amount_applied'] = value.get_amount_applied()
            apply_credits.append(credits)
        data['invoice_payments'] = invoice_payments
        data['apply_creditnotes'] = apply_credits
        json_string = {
            'JSONString': dumps(data)
            }
        response = zoho_http_client.post(url, self.details, json_string)
        return parser.apply_credits(response) 

    def delete_payment(self, invoice_id, invoice_payment_id):
        """Delete a payment made to an invoice.

        Args:
            invoice_id(str): Invoice id.
            invoice_payment_id(str): Invoice payment id.

        Returns:
            str: Success message('The payment has been deleted.').
         
        """
        url = base_url + invoice_id + '/payments/' + invoice_payment_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 

    def delete_applied_credit(self, invoice_id, creditnotes_invoice_id):
        """Delete a particular credit applied to an invoice.

        Args:
            invoice_id(str): Invoice id.
            creditnotes_invoice_id(str): Creditnotes invoice id.
      
        Returns:
            str: Success message('Credits applied to an invoice have been 
                deleted.').

        """
        url = base_url + invoice_id + '/creditsapplied/' + \
              creditnotes_invoice_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 
        
## Attachment------------------------------------------------------------------

    def get_an_invoice_attachment(self, invoice_id, preview = None):
        """Get an invoice attachment.

        Args:
            invoice_id(str): Invoice id
            preview(bool): True to get the thumbnail of the attachment.
        Returns:
            file: File attached to the invoice.
 
        """
        url = base_url + invoice_id + '/attachment'
        query_string = {
                'preview': str(preview)
                } if preview is not None else None     
        response = zoho_http_client.getfile(url, self.details, query_string)
        return response  

    def add_attachment_to_an_invoice(self, invoice_id, attachment, \
        can_send_in_mail=None):
        """Add a file to an invoice.

        Args:
            invoice_id(str): Invoice id.
            attachment(list of dict): List of dict containing details of the 
                files to be attached.
            can_send_in_mail(bool, optional): True to send the attachment with 
                the invoice when emailed.

        Returns:
            str: Success message('Your file has been successfully attached to 
                the invoice.').

        """
        url = base_url + invoice_id + '/attachment'
        file_list = []
        for value in attachment:
            attachments = {
                'attachment': {
                    'filename':basename(value), 
                    'content':open(value).read()
                    }
                }
            file_list.append(attachments)
            if can_send_in_mail is not None:
                query = {
                    'can_send_in_mail':can_send_in_mail
                    }
            else:
                query = None
        data = {
            'JSONString': ''
            }
        response = zoho_http_client.post(url, self.details, data, query, \
                                         file_list)
        return parser.get_message(response) 

    def update_attachment_preference(self, invoice_id, can_send_in_mail):
        """Update whether to send attached file while emailing the invoice.
        
        Args:
            invoice_id(str): Invoice id.
            can_send_in_mail(bool): Boolean to send attachment with the 
                invoice when emailed.
        
        Returns:
            str: Success message('Invoice information has been updated.').
        
        """     
        url = base_url + invoice_id + '/attachment'
        query = {
            'can_send_in_mail':can_send_in_mail, 
            }
        data = {
            'JSONString': ''
            }
        response = zoho_http_client.put(url, self.details, data, query)
        return parser.get_message(response) 

    def delete_an_attachment(self, invoice_id):
        """Delete the file attached to the invoice.

        Args:
            invoice_id(str): Invoice id.

        Returns:
            str: Success message('Your file is no longer attached 
                to the invoice.').

        """
        url = base_url + invoice_id + '/attachment'
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 
 
    def delete_expense_receipt(self, expense_id):
        """Delete the expense receipts attached to an invoice which 
            is raised from an expense.

        Args:
            expense_id: Expense id.
            
        Returns:
            str: Success message('The attached expense receipt has 
                been deleted.').
  
        """
        url = base_url + 'expenses/' + expense_id + '/receipt'
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 

  
#### Comments and History -----------------------------------------------------



    def list_invoice_comments_history(self, invoice_id):
        """Get the complete history and comments of an invoice.

        Args:
            invoice_id(str): Invoice_id.
         
        Returns:
            instance: Comments list object.

        """ 
        url = base_url + invoice_id + '/comments'
        response = zoho_http_client.get(url, self.details)
        return parser.comments_list(response) 
  
    def add_comment(self, invoice_id, comments):
        """Add comment for an invoice.
 
        Args:
            invoice_id(str): Invoice id.
            comments(instance): Comments object.
 
        Returns:
            str: Success message('Comments added.').

        """
        url = base_url + invoice_id + '/comments'
        data = {}
        data['payment_expected_date'] = comments.get_payment_expected_date()
        data['description'] = comments.get_description()
        data['show_comment_to_clients'] = \
        comments.get_show_comment_to_clients()
        json_string = {
            'JSONString': dumps(data)
            }
        response = zoho_http_client.post(url, self.details, json_string)
        return parser.get_comment(response) 

    def update_comment(self, invoice_id, comment_id, comments):
        """Update an existing comment of an invoice.

        Args:
            invoice_id(str): Invoice id.
            comment_id(str): Comment id.
            comments(instance): Comments object.
        
        Returns:
            instance: Comments object.

        """
        url = base_url + invoice_id + '/comments/' + comment_id
        data = {}
        data['description'] = comments.get_description()
        data['show_comment_to_clients'] = \
        comments.get_show_comment_to_clients()
        json_string = {
            'JSONString': dumps(data)
            }
        response = zoho_http_client.put(url, self.details, json_string)
        return parser.get_comment(response) 
  
    def delete_comment(self, invoice_id, comment_id):
        """Delete an invoice comment.

        Args:
            invoice_id(str): Invoice id.
            comment_id(str): Comment id.

        Returns:
            str: Success message.('The comment has been deleted.').

        """
        url = base_url + invoice_id + '/comments/' + comment_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 
   
 



#$Id$

class PaymentAndCredit:
    """This class is used to create object for Payments and Credits."""
    def __init__(self): 
        """Initialize parameters for payments and credits."""
        self.invoice_payments=[]
        self.apply_creditnotes=[]

    
    def set_payments(self,payments):
        """Set payments.

        Args:
            payments(instace): Payemnts object.

        """
        self.invoice_payments.append(payments)

    def get_payments(self):
        """Get payments.

        Returns:
            instance: Payments object.

        """
        return self.invoice_payments

    def set_apply_creditnotes(self,credits):
        """Set apply creditnotes.
 
        Args:
            credits(instance): Credits object.

        """
        self.apply_creditnotes.append(credits)

    def get_apply_creditnotes(self):
        """Get apply creditnotes.

        Returns:
            instance: Credits object.
        
        """
        return self.apply_creditnotes

   



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.SettingsParser import SettingsParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'users/'
zoho_http_client = ZohoHttpClient()
parser = SettingsParser()

class UsersApi:
    """Uses Api class is used to 
 
    1.List of all users in an organization.
    2.10.Get the details of an user.
    3.Get the details of the current user.
    4.Create an user for an organization.
    5.Update the details of an existing user.
    6.Delete an existing user.
    7.Send invitation email to a user.
    8.Mark an inactive user as active.
    9.Mark an active user as inactive.
    
    """
    def __init__(self, authtoken, organization_id):
        """Initialize Settings Api using authtoken and organization id.

        Args:
            authtoken(str): User's Authtoken.
            organization_id(str): User's Organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id 
            }
    def get_users(self, parameters=None):
        """Get the list of users in the organization.

        Args:
            parameters(dict, optional): Filter with which the list has to be 
                displayed.

        Returns: 
            instance: Users list object.

        """
        resp = zoho_http_client.get(base_url, self.details, parameters)
        return parser.get_users(resp) 

    def get(self, user_id):
        """Get the details of a user.

        Args:
            user_id(str): User id.
  
        Returns:
            instance: User object.

        """
        url = base_url + user_id
        resp = zoho_http_client.get(url, self.details) 
        return parser.get_user(resp) 

    def current_user(self):
        """Get details of a user.

        Returns:
            instance: User object.

        """
        url = base_url + 'me'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_user(resp) 

    def create(self, user):
        """Create a user for your organization.

        Args:
            user(instance): User object.

        Returns: 
            instance: User object.

        """
        json_obj = dumps(user.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.post(base_url, self.details, data)
        return parser.get_user(resp) 

    def update(self, user_id, user):
        """Update the details of an user.

        Args:
            user_id(str): User id.
            user(instance): User object.

        Returns:
            instance: User object.

        """
        url = base_url + user_id
        json_obj = dumps(user.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_user(resp) 

    def delete(self, user_id):
        """Delete a user associated to the organization.

        Args:
            user_id(str): User id.

        Returns:
            str: Success message('The user has been removed from your 
                organization.').

        """
        url = base_url + user_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def invite_user(self, user_id):
        """Send invitation email to a user.

        Args:
            user_id(str): User id.

        Returns:
            str: Success message('Your invitation has been sent.').
 
        """
        url = base_url + user_id + '/invite'
        data = { 
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp) 

    def mark_user_as_active(self, user_id):
        """Mark an inactive user as active.

        Args:
            user_id(str): User id.

        Returns:
            str: Success message('The user has been marked as active.').
 
        """
        url = base_url + user_id + '/active'
        data = { 
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp) 
 
    def mark_user_as_inactive(self, user_id):
        """Mark an active user as inactive.

        Args:
            user_id(str): User id.
 
        Returns:
            str: Success message('The user has been marked as inactive.').
      
        """
        url = base_url + user_id + '/inactive'
        resp = zoho_http_client.post(url, self.details, '')
        return parser.get_message(resp) 



#$Id$

class Address:
    """This class is used to create an object for Address."""
    def __init__(self):
        """Initialize parameters for address object"""
        self.address = ''
        self.city = ''
        self.state = ''  
        self.zip = ''
        self.country = ''
        self.fax = ''
        self.is_update_customer = None
        self.street_address1 = ''
        self.street_address2 = ''

    def set_address(self, address):
        """Set address.
        
        Args:
            address(str): Address.
         
        """
        self.address = address

    def get_address(self):
        """Get address.
        
        Returns:
            str: Address.

        """
        return self.address
  
    def set_city(self, city):
        """Set city.

        Args:
            str: City.
         
        """
        self.city = city

    def get_city(self):
        """Get city.
    
        Returns:
            str: City.
        
        """
        return self.city
  
    def set_state(self, state):
        """Set state.
        
        Args:
            state(str): State.

        """
        self.state = state

    def get_state(self):
        """Get the state.
        
        Returns:
            str: State.
        
        """
        return self.state
  
    def set_zip(self, zip_code):
        """Set zip.
 
        Args:
            zip_code(str): Zip code.
        
        """
        self.zip = zip_code
 
    def get_zip(self):
        """Get zip.
     
        Returns:
            str: Zip code.

        """
        return self.zip
  
    def set_country(self, country):
        """Set country.
        
        Args:
            country(str): Country.
     
        """
        self.country = country
     
    def get_country(self):
        """Get country.
     
        Returns:
            str: Country.
        
        """
        return self.country
  
    def set_fax(self, fax):
        """Set fax.
       
        Args:
            fax(str): Fax.
      
        """
        self.fax = fax

    def get_fax(self):
        """Get fax.
 
        Returns:
            str: Fax.
      
        """  
        return self.fax
 
    def set_is_update_customer(self, is_update_customer):
        """Set whether to update customer.
        
        Args:
            is_update_customer(bool): True to update customer else False.
         
        """
        self.is_update_customer = is_update_customer

    def get_is_update_customer(self):
        """Get is update customer
      
        Returns:
            bool: True to update customer else False.
        
        """
        return self.is_update_customer

    def set_street_address1(self, street_address1):
        """Set street address1.

        Args:
            street_address1(str): Street address 1.

        """
        self.street_address1 = street_address1

    def get_street_address1(self):
        """Get street address1.

        Returns:
            str: Street address 1.

        """
        return self.street_address1

    def set_street_address2(self, street_address2):
        """Set street address 2.

        Args:
            street_address2(str): street address 2.

        """
        self.street_address2 = street_address2

    def get_street_address2(self):
        """Get street address 2.

        Returns:
            str: Street address 2.

        """
        return self.street_address2

    def to_json(self):
        """This method is used to convert the address object to JSON object.

        Returns:
            dict: Dictionary containing details of address object.

        """
        address = {}
        if self.street_address1 != '':
            address['street_address1'] = self.street_address1
        if self.street_address2 != '':
            address['street_address2'] = self.street_address2
        if self.address != '':
            address['address'] = self.address
        if self.city != '':
            address['city'] = self.city
        if self.state != '':
            address['state'] = self.state
        if self.zip != '':
            address['zip'] = self.zip
        if self.country != '':  
            address['country'] = self.country
        if self.fax != '':
            address['fax'] = self.fax
        return address



#$Id$

class ChartOfAccount:
    """Create object for Chart of Accounts."""
    def __init__(self):
        """Initialize parameters for Chart of accounts."""
        self.account_id = ''
        self.account_name = ''
        self.is_active = None
        self.account_type = ''
        self.account_type_formatted = ''
        self.currency_id = ''
        self.description = ''
        self.is_user_created = None
        self.is_involved_in_transaction = None
        self.is_system_account = None
        #self.current_balance = 0.0 #Not mentioned 

    def set_account_id(self, account_id):
        """Set account id.

        Args:
            account_id(str): Account id.

        """
        self.account_id = account_id

    def get_account_id(self): 
        """Get account id.

        Returns: 
            str: Account id.

        """ 
        return self.account_id
  
    def set_account_name(self, account_name):
        """Set account name.

        Args: 
            account_name(str): Account name.
 
        """
        self.account_name = account_name

    def get_account_name(self): 
        """Get account name.

        Returns:
            str: Account name.

        """
        return self.account_name

    def set_is_active(self, is_active): 
        """Set whether the account is active or not.

        Args:
            is_active(bool): True if active else False.

        """
        self.is_active = is_active

    def get_is_active(self): 
        """Get whether is active.

        Returns:
            bool: True if active else false.

        """
        return self.is_active

    def set_account_type(self, account_type):
        """Set account type.

        Args:
            account_type(str): Account type.

        """
        self.account_type = account_type

    def get_account_type(self):
        """Get account type.

        Returns:
            str: Account type.

        """
        return self.account_type

    def set_account_type_formatted(self, account_type_formatted):
        """Set account type formatted.

        Args:
            account_type_formatted(str): Account type formatted.

        """
        self.account_type_formatted = account_type_formatted

    def get_account_type_formatted(self):
        """Get acccount type formatted.

        Returns:
            str: Account type formatted.

        """
        return self.account_type_formatted
  
    def set_currency_id(self, currency_id): 
        """Set currency id.

        Args:
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self): 
        """Get currency id.

        Returns:
            str: Currency id.

        """
        return self.currency_id

    def set_description(self, description):
        """Set descripiton.

        Args:
            description(str): Description.

        """
        self.description = description

    def get_description(self):
        """Get description.

        Returns:
            str: Description.

        """
        return self.description

    def set_is_user_created(self, is_user_created):
        """Set whether the account is user created.

        Args:
            is_user_created(bool): True if user created else False.

        """
        self.is_user_created = is_user_created

    def get_is_user_created(self):
        """Get whether the account is user created.

        Returns:
            bool: True if user created else False.

        """
        return self.is_user_created

    def set_is_involved_in_transaction(self, is_involved_in_transaction):
        """Set Whether the account is involved in transactions.

        Args:
            is_involved_in_transaction(bool): True if the account is involved in transactions else False.

        """
        self.is_involved_in_transaction = is_involved_in_transaction

    def get_is_involved_in_transaction(self):
        """Get whether the account is involved in transactions.

        Returns:
            bool: True if the account is involved in transaction.

        """
        return self.is_involved_in_transaction

    def set_is_system_account(self, is_system_account): 
        """Set whether the account is system account.

        Args:
            is_system_account(bool): True if system account else False.

        """
        self.is_system_account = is_system_account

    def get_is_system_account(self):
        """Get whether the account is system account.

        Returns:
            bool: True if system account else False.

        """
        return self.is_system_account

    '''def set_current_balance(self, current_balance):
        """Set current balance.

        Args:
            current_balance(float): Current balance.

        """
        self.current_balance = current_balance

    def get_current_balance(self): 
        """Get current balance.

        Returns:
            float: Current balance.

        """
        return self.current_balance'''

    def to_json(self):
        """This method is used to convert chart of accounts object to json object.

        Returns:
            dict: Dictionary containing json object for chart of accounts.

        """
        data = {}
        if self.account_name != '':
            data['account_name'] = self.account_name
        if self.account_type != '':
            data['account_type'] = self.account_type
        if self.currency_id != '':
            data['currency_id'] = self.currency_id
        if self.description != '':
            data['description'] = self.description
        return data

 



#$Id$

from books.model.PageContext import PageContext

class TaskList:
    """This class is used to create object for tasks list."""
    def __init__(self):
        """Initialize parameters for tasks list object."""
        self.tasks = []
        self.page_context = PageContext()
 
    def set_tasks(self, task):
        """Set tasks.

        Args:
            task(instance): Task object.

        """
        self.tasks.append(task)

    def get_tasks(self): 
        """Get tasks.

        Returns: 
            list of instance: List of tasks object.

        """
        return self.tasks

    def set_page_context(self, page_context): 
        """Set page context.

        Args:
            page_context(instance): Page context. 

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context.

        """
        return self.page_context



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.CustomerPaymentsParser import CustomerPaymentsParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'customerpayments/'
parser = CustomerPaymentsParser()
zoho_http_client = ZohoHttpClient()

class CustomerPaymentsApi:
    """CustomerPaymentsApi class is used:
     
    1.To list all the payments made by the customer.
    2.To get the details of the customer payment.
    3.To create a payment made by the customer.
    4.To update a payment made by the customer.
    5.To delete an existing customer payment.

    """

    def __init__(self, authtoken, organization_id):
        """Initialize Customer payment's api using user's authtoken 
               and organization id.
        
        Args: 
            authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details={
            'authtoken': authtoken,
            'organization_id': organization_id
            }
 
    def get_customer_payments(self, parameter=None):
        """List all customer payments with pagination.
 
        Args:
            parameter(dict, optional): Filter with which the list has 
                to be displayed. Default to None.

        Returns:
            instance: Customer payments list object.
        
        """
        response = zoho_http_client.get(base_url, self.details, parameter) 
        return parser.customer_payments(response)

    def get(self, payment_id):
        """Get details of a customer payment.

        Args:
            payment_id(str): Payment id of the customer.

        Returns:
            instance: Customer payment object.

        """
        url = base_url + payment_id
        response = zoho_http_client.get(url, self.details)
        return parser.get_customer_payment(response)

    def refund(self, payment_id, refund):
        """Refund excess of a customer payment.

        Args:
            payment_id(str): Payment id of the customer.
            refund(CreditNoteRefund): Refund the amount to customer.

        Returns:
            instance: Customer payment object.

        """
        url = base_url + payment_id + '/refunds'
        data = refund.to_json()
        field = {
            'JSONString': dumps(data)
        }
        response = zoho_http_client.post(url, self.details, field, None)
        return parser.payment_refunds(response)

    def get_refunds(self, payment_id):
        """List Refunds of a customer payment.

        Args:
            payment_id(str): Payment id of the customer.

        Returns:
            instance: Customer payment object.

        """
        url = base_url + payment_id + '/refunds'
        response = zoho_http_client.get(url, self.details, None)
        return parser.payment_refunds(response)

    def create(self, customer_payment):
        """Create a payment made by the customer.

        Args:
            customer_payment(instance): customer payment object.

        Returns:
            instance: customer payment object.

        """
        json_object = dumps(customer_payment.to_json())
        data = {
            'JSONString': json_object
            }
        response = zoho_http_client.post(base_url,self.details,data)
        return parser.get_customer_payment(response) 
  
    def update(self, payment_id, customer_payment):
        """Update a payment made by a customer.
        
        Args:
            payment_id(str): Payment id.
            customer_payment(instance): Customer payment object.
       
        Returns:
            instance: Customer payment object.
 
        """
        url = base_url + payment_id
        json_object = dumps(customer_payment.to_json())
        data = {
            'JSONString': json_object
            }
        response = zoho_http_client.put(url,self.details,data)
        return parser.get_customer_payment(response) 

    def delete(self, payment_id):
        """Delete an existing customer payment.

        Args:
            payment_id(str): Payment id.
 
        Returns:
            str: Success message ('The payment has been deleted.')
      
        """
        url = base_url + payment_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 




#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.BankRulesParser import BankRulesParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'bankaccounts/rules/'
zoho_http_client = ZohoHttpClient()
parser = BankRulesParser()

class BankRulesApi:
    """This class is used to 

    1.Fetch all the rules created for a specified bank or credit card account.
    2.Get details of a specific rule.
    3.Create a rule.
    4.Update an existing rule.
    5.Delete a rule.

    """
    def __init__(self, authtoken, organization_id):
        """Initialize Bank rules Api using user's authtoken and organization 
            id.

        Args: 
            authotoken(str): User's Authtoken.
            organization id(str): User's Organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id
            }
    
    def get_rules(self, account_id):
        """Get list of rules.

        Args:
            account_id(str): Account id for which the rules have to be listed.
        
        Returns: 
            instance: Bank rules list object.

        """
        param = {
            'account_id': account_id
            }
        resp = zoho_http_client.get(base_url, self.details, param)
        return parser.get_rules(resp) 

    def get(self, rule_id):
        """Get details of a rule.
 
        Args:
            rule_id(str): Rule id.

        Returns:
            instance: Bank rules object.

        """
        url = base_url + rule_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_rule(resp) 

    def create(self, rule):
        """Create a rule.

        Args:
            rule(instance): Bank rule object.

        Returns:
            instance: Bank rule object.

        """
        json_object = dumps(rule.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(base_url, self.details, data)
        return parser.get_rule(resp)
   
    def update(self, rule_id, rule):
        """Update an existing rule.

        Args:
            rule_id(str): Rule id.

        Returns:
            instance: Bank rule object.

        """
        url = base_url + rule_id
        json_object = dumps(rule.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_rule(resp) 

    def delete(self, rule_id):
        """Delete an existing rule.

        Args:
            rule_id(str): Rule id.

        Returns:
            str: Success message('The rule has been deleted.').

        """
        url = base_url + rule_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
      



#$Id$

from books.model.PageContext import PageContext

class ContactList:
    """This class is used to create an object for contact list."""
    def __init__(self):
        """ Initialize contacts list and page context."""
        self.contacts = []
        self.page_context = PageContext()

    def set_contacts(self, contact):
        """Insert contact object to contacts list.
        
        Args:
            contact(instance): Contact object.
   
        """
        self.contacts.append(contact)

    def get_contacts(self):
        """Get contacts list.
        
        Returns:
            list: List of contacts object.
        
        """
        return self.contacts
 
    def set_page_context(self, page_context):
        """Set page context.
        
        Args:
            instance: Page_context object.
        
        """
        self.page_context = page_context
     
    def get_page_context(self):
        """Get page context.
        
        Returns:
            instance: Page context object.
       
        """
        return self.page_context





#$Id$#

class InvoiceCreditedList:
    """This class is used to create object for invoice credited list."""
    
    def __init__(self):
        """Initialize parameters for invoice credited list object."""
        self.invoices_credited = [] 

    def set_invoices_credited(self, invoice_credited):
        """Set invoices credited.

        Args:
            invoice_credited(instance): Invoice credited.

        """
        self.invoices_credited.append(invoice_credited)

    def get_invoices_credited(self):
        """Get invoices credited.

        Returns:
            list of instance: List of invoice credited.

        """
        return self.invoices_credited



#$Id$

class AutoReminderList:
    """This class is used to create object for Reminder list."""
    def __init__(self):
        """Initialize parameters for Reminder list."""
        self.auto_reminders = []

    def set_auto_reminders(self, reminder):
        """Set reminder.

        Args:
            reminder(instance): Reminder object.

        """
        self.auto_reminders.append(reminder)

    def get_auto_reminders(self):
        """Get reminders.

        Returns:
            list of instance: List of manual reminders object.

        """
        return self.auto_reminders



#$Id$

class PaymentList:
    """This class is used to create object for payment list."""
    def __init__(self):
        """Initialize parameters for Payment list object."""
        self.payments = []

    def set_payments(self, payment):
        """Set payments.

        Args: 
            payment(instance): Payment object.

        """
        self.payments.append(payment)
   
    def get_payments(self):
        """Get payment.

        Returns:
            list of instance: List of payment object.

        """
        return self.payments



#$Id$#

from os.path import basename
from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.EstimatesParser import EstimatesParser
from .Api import Api
from json import dumps

base_url = Api().base_url + 'estimates/'
parser = EstimatesParser()
zoho_http_client = ZohoHttpClient()

class EstimatesApi:
    """Estimates Api class is used to:
   
    1.List all estimates with pagination.
    2.Get the details of an estimate.
    3.Create an estimate.
    4.Update an existing estimate.
    5.Delete an existing estimate.
    6.Mark a draft estimate as sent.
    7.Mark a sent estimate as accepted.
    8.Mark a sent estimate as declined.
    9.Email an estimate to the customer.
    10.Send estimates to your customer by email.
    11.Get the email content of an estimate.
    12.Export maximum of 25 pdfs as single pdf.
    13.Export estimates as pdf and print them.
    14.Update the billing address for the estimate.
    15.Update the shipping address for the estimate.
    16.Get all estimate pdf templates.
    17.Update the pdf template associated with the template.
    18.Get the complete history and comments of an estimate.
    19.Add a comment for an estimate.
    20.Update an existing comment of an estimate.
    21.Delete an estimate comment.

    """
    def __init__(self, authtoken, organization_id):
        """Initialize Estimates api using user's authtoken and 
            organization id.

        Args:
            authtoken(str): Authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details = {
            'authtoken':authtoken, 
            'organization_id':organization_id
            }
  
    def get_estimates(self, parameter=None):
        """Get list of estimates with pagination.

        Args:
            parameter(dict, optional): Filter with which the list has to be 
                dispalyed.

        Returns:
            instance: Invoice list object.

        """  
        response = zoho_http_client.get(base_url, self.details, parameter)
        return parser.get_list(response) 

    def get(self, estimate_id, print_pdf=None, accept=None):   
        """Get details of an estimate.
 
        Args:
            estimate_id(str): Estimate id.
            print_pdf(bool, None): True to print pdf else False.
            accept(str, None): Get the details in particular format such as 
                json/pdf/html. Default format is json. Allowed values are json,
                pdf and html.

        Returns: 
            instance: Estimate object.
        
        """
        url = base_url + estimate_id
        query = {}
        if print_pdf is not None and accept is not None:
            query = {
                'print':print_pdf, 
                'accept':accept
                }
            resp = zoho_http_client.getfile(url, self.details, query)
            return resp
        elif print_pdf is True:
            query = {
                'print':print_pdf, 
                'accept':'pdf'
                } 
            resp = zoho_http_client.getfile(url, self.details, query)
            return resp
        elif accept is not None:
            query = {
                'accept':accept
                }
            resp = zoho_http_client.getfile(url, self.details, query)
            return resp 
        else:
            response = zoho_http_client.get(url, self.details)
            return parser.get_estimate(response) 

    def create(self, estimate, send=None, ignore_auto_number_generation=None):
        """Create an estimate.

        Args:
            estimate(instance): Estimate object.
            send(bool, optional): True to send estimate to the contact persons 
                associated with the estimate. Allowed values are true and false.
            ignore_auto_number_generation(bool, optional): True to ignore auto 
                estimate number generation for this estimate. Allowed values 
                are true and false.

        Returns:
            instance: Invoice object.

        """
        query = {}
        if send is not None and ignore_auto_number_generation is not None:
           query = {
               'send':send,  
               'ignore_auto_number_generation':ignore_auto_number_generation
               }
        elif send is not None or ignore_auto_number_generation is not None:
             query = {
                 'send': send
                 } if send is not None else {
                 'ignore_auto_number_generation': ignore_auto_number_generation
                 }
        else:
            query = None
        json_object = dumps(estimate.to_json())
        data = {
            'JSONString': json_object
            }
        response = zoho_http_client.post(base_url, self.details, data, query)
        return parser.get_estimate(response)     

    def update(self, estimate_id, estimate, 
               ignore_auto_number_generation=None):
        """Update an existing estimate.
 
        Args:
            estimate_id(str): Estiamte id.
            estiamte(instance): Estiamte object.
            ignore_auto_number_generation(bool, optional): True to ignore auto 
                estimate number generation for this estimate. Allowed values 
                are true and false.

        Returns:
            instance: Estimates object.

        """
        url = base_url + estimate_id   
        query = {}
        if ignore_auto_number_generation is not None:
            query = {
                'ignore_auto_number_generation': ignore_auto_number_generation
                }
        else:
            query = None
        json_object = dumps(estimate.to_json())
        data = {
            'JSONString': json_object
            }
        response = zoho_http_client.put(url, self.details, data) 
        return parser.get_estimate(response)
    
    def delete(self, estimate_id):
        """Delete an existing estimate.

        Args:
            estimate_id(str): Estimate id.

        Returns:
            str: Success message('The estimate has been deleted.').

        """ 
        url = base_url + estimate_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response)

    def mark_an_estimate_as_sent(self, estimate_id):
        """Mark a draft estimate as sent.

        Args:
            estimate_id(str): Estimate id.

        Returns:
            str: Success message('Estimate status has been changed to Sent.').

        """
        url = base_url + estimate_id + '/status/sent'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 

    def mark_an_estimate_as_accepted(self, estimate_id):
        """Mark a sent estimate as accepted.

        Args: 
            estimate_id(str): Estimate id.

        Returns:
            str: Success message('Estimate status has been changed to sent.').
 
        """
        url = base_url + estimate_id + '/status/accepted'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 
   
    def mark_an_estimate_as_declined(self, estimate_id):
        """Mark a sent estimate as declined.

        Args:
            estimate_id(str): Estiamte id.

        Returns:
            str: Success message('Estimate status has been changed to 
                accepted.').

        """
        url = base_url + estimate_id + '/status/declined'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 
  
    def email_an_estimate(self, estimate_id, email, attachment=None):
        """Email an estimate to the customer.

        Args:
            estimate_id(str): Estimate id.
            email(instance): Email object. 
            attachment(list of dict): List of dictionary containing details 
                about files to be attached.

        Returns:
            instance: Email object.

        """
        url = base_url + estimate_id + '/email'
        json_object = dumps(email.to_json())
        data = {
            'JSONString': json_object
            }
        if attachment is not None:
            file_list = []
            for value in attachment:
                attachments = {
                    'attachments': {
                        'filename': basename(value), 
                        'content':open(value).read()
                        } 
                }
                file_list.append(attachments)
        else:
            file_list = None
        response = zoho_http_client.post(url, self.details, data, None, \
                   file_list) 
        return parser.get_message(response) 
  
    def email_estimates(self, estimate_id):
        """Email estimates to the customer.

        Args: 
            estimate_id(str): Estimate id.

        Returns: 
            str: Success message('Mission accomplished.').

        """
        url = base_url + 'email'
        estimate_ids = {
            'estimate_ids': estimate_id
            }
        response = zoho_http_client.post(url, self.details, '', \
                   estimate_ids)
        return parser.get_message(response) 

    def get_estimate_email_content(self, estimate_id, email_template_id=None):
        """Get estimate email content.

        Args:
            estiamte_id(str): Estiamte id.
            email_template_id(str): Email template id.

        Returns:
            instance: Email object.

        """
        url = base_url + estimate_id + '/email'
        query = {}
        if email_template_id is not None:
            query = {
                'email_template_id':email_template_id
                }
        else:
            query = None
        response = zoho_http_client.get(url, self.details, query)
        return parser.get_email_content(response) 
 
    def bulk_export_estimates(self, estimate_id):
        """Export maximum of 25 estimates as pdf.

        Args:
            estimate_id(str): Estimate id.

        Returns:
            file: Pdf containing details of estimate.

        """
        url = base_url + 'pdf'
        query = {
            'estimate_ids': estimate_id
            }
        response = zoho_http_client.getfile(url, self.details, query)
        return response 
  
    def bulk_print_estimates(self, estimate_ids):
        """Export estimates as pdf and print them.

        Args: 
            estimate_ids(str): Estimate ids.

        Returns:
            file: Pdf containing details of estimate.

        """
        url = base_url + 'print'
        query = {
            'estimate_ids': estimate_ids
            }
        response = zoho_http_client.getfile(url, self.details, query)
        return response

   
    def update_billing_address(self, estimate_id, address, 
                               is_update_customer=None):
        """Update billing address.

        Args:
            estimate_id(str): Estiamte id.
            address(instance): Address object.
            is_update_customer(bool, optional): True to update customer.

        Returns:
            instance: Address object.

        """
        url = base_url + estimate_id + '/address/billing' 
        json_object = dumps(address.to_json())
        data = {
            'JSONString': json_object
            }
        if is_update_customer is not None:
            query = {
                'is_update_customer':is_update_customer
                }
        else: 
            query = None
        response = zoho_http_client.put(url, self.details, data, query)
        return parser.get_billing_address(response) 

    def update_shipping_address(self, estimate_id, address, 
                                is_update_customer=None):
        """Update shipping address.

        Args: 
            estimate_id(str): Estimate id.
            address(instance): Address object.
            is_update_customer(bool, optional): True to update customer 
                else False.

        Returns:
            instance: Address object.

        """
        url = base_url + estimate_id + '/address/shipping'
        json_object = dumps(address.to_json())
        data = {
            'JSONString': json_object
            }
        if is_update_customer is not None:
            query = {
                'is_update_customer': is_update_customer
                }
        else:
            query = None
        response = zoho_http_client.put(url, self.details, data, query)
        return parser.get_shipping_address(response)
 
    def list_estimate_template(self):
        """List all estimate pdf templates.

        Returns:
            instance: Template list object.

        """
        url = base_url + 'templates'
        response = zoho_http_client.get(url, self.details)
        return parser.estimate_template_list(response)
  
    def update_estimate_template(self, estimate_id, template_id):
        """Update estimate template.

        Args:
            estimate_id(str): Estimate id.
            template_id(str): Template id.

        Returns:
            str: Success message('Estimate information has been updated.').
        
        """
        url = base_url + estimate_id + '/templates/' + template_id
        response = zoho_http_client.put(url, self.details, '')
        return parser.get_message(response)

# Comments and History

    def list_comments_history(self, estimate_id):
        """Get complete history and comments.

        Args:
            estiamte_id(str): Estimate id.

        Returns:
            list of instance: list of comments object.
 
        """ 
        url = base_url + estimate_id + '/comments'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_comments(resp)
  
    def add_comment(self, estimate_id, description, show_comment_to_clients):
        """Add a comment for an estimate.

        Args:
            estimate_id(str): Estimate id.
            description(str): Description.
            show_comment_to_clients(bool): True to show comment to clients.

        Returns:
            instance: Comments object.

        """
        url = base_url + estimate_id + '/comments'
        data = {}
        data['description'] = description
        data['show_comment_to_clients'] = show_comment_to_clients
        field = {
            'JSONString': dumps(data)
            }
        resp = zoho_http_client.post(url, self.details, field)
        return parser.get_comment(resp)

    def update_comment(self, estimate_id, comment_id, description, 
                       show_comment_to_clients):
        """Update an existing comment.

        Args:
            estimate_id(str): Estimate id.
            comment_id(str): Comment id.
            description(str): Description.
            show_comments_to_clients(bool): True to show comments to clients.
         
        Returns:
            instance: Comments object.
       
        """
        url = base_url +estimate_id + '/comments/' + comment_id
        data = {} 
        data['description'] = description
        data['show_comment_to_clients'] = show_comment_to_clients
        field = {
            'JSONString': dumps(data)
            }
        resp = zoho_http_client.put(url, self.details, field)
        return parser.get_comment(resp) 
  
    def delete_comment(self, estimate_id, comment_id):
        """Delete an existing comment.

        Args:
            comment_id(str): Comment id.

        Returns:
            str: Success message('The comment has been deleted.').
       
        """
        url = base_url + estimate_id + '/comments/' + comment_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
   
   



#$Id$#

from books.model.RecurringInvoice import RecurringInvoice
from books.model.RecurringInvoiceList import RecurringInvoiceList
from books.model.PageContext import PageContext
from books.model.LineItem import LineItem
from books.model.CustomField import CustomField
from books.model.Address import Address
from books.model.PaymentGateway import PaymentGateway
from books.model.Comment import Comment
from books.model.CommentList import CommentList

class RecurringInvoiceParser:
    """This class is used to parse the json for Recurring invoice."""
    def recurring_invoices(self, response):
        """This method parses the given response and creates recurring 
            invoices list object.
      
        Args:
            response(dict): Response containing json object for recurring 
                invoice list.
 
        Returns:
            instance: Recurring invoice list object.

        """
        recurring_invoices_list = RecurringInvoiceList()
        recurring_invoices = []
        for value in response['recurring_invoices']:
            recurring_invoice = RecurringInvoice()
            recurring_invoice.set_recurring_invoice_id(\
            value['recurring_invoice_id'])
            recurring_invoice.set_recurrence_name(value['recurrence_name'])
            recurring_invoice.set_status(value['status'])
            recurring_invoice.set_total(value['total'])
            recurring_invoice.set_customer_id(value['customer_id'])
            recurring_invoice.set_customer_name(value['customer_name'])
            recurring_invoice.set_start_date(value['start_date'])
            recurring_invoice.set_end_date(value['end_date'])
            recurring_invoice.set_last_sent_date(value['last_sent_date'])
            recurring_invoice.set_next_invoice_date(value['next_invoice_date'])
            recurring_invoice.set_recurrence_frequency(\
            value['recurrence_frequency'])
            recurring_invoice.set_repeat_every(value['repeat_every'])
            recurring_invoice.set_created_time(value['created_time'])
            recurring_invoice.set_last_modified_time(\
            value['last_modified_time'])
            recurring_invoices.append(recurring_invoice)
        recurring_invoices_list.set_recurring_invoices(recurring_invoices)

        page_context = response['page_context']
        page_context_obj = PageContext()
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        recurring_invoices_list.set_page_context(page_context_obj)

        return recurring_invoices_list

    def recurring_invoice(self, response):
        """This method parses the response and creates recurring invoice 
            object.
  
        Args:
            response(dict): Response containing json object for recurring 
                invoice.

        Returns:
            instance: Recurring invoice object.

        """
        recurring_invoice = response['recurring_invoice']
        recurring_invoice_obj = RecurringInvoice()
        recurring_invoice_obj.set_recurring_invoice_id(\
        recurring_invoice['recurring_invoice_id'])
        recurring_invoice_obj.set_recurrence_name(\
        recurring_invoice['recurrence_name'])
        recurring_invoice_obj.set_status(recurring_invoice['status'])
        recurring_invoice_obj.set_recurrence_frequency(\
        recurring_invoice['recurrence_frequency'])
        recurring_invoice_obj.set_repeat_every(\
        recurring_invoice['repeat_every'])
        recurring_invoice_obj.set_start_date(recurring_invoice['start_date'])
        recurring_invoice_obj.set_end_date(recurring_invoice['end_date'])
        recurring_invoice_obj.set_last_sent_date(\
        recurring_invoice['last_sent_date'])
        recurring_invoice_obj.set_next_invoice_date(\
        recurring_invoice['next_invoice_date'])
        recurring_invoice_obj.set_payment_terms(\
        recurring_invoice['payment_terms'])
        recurring_invoice_obj.set_payment_terms_label(\
        recurring_invoice['payment_terms_label'])
        recurring_invoice_obj.set_customer_id(recurring_invoice['customer_id'])
        recurring_invoice_obj.set_customer_name(\
        recurring_invoice['customer_name'])
        recurring_invoice_obj.set_contact_persons(\
        recurring_invoice['contact_persons'])
        recurring_invoice_obj.set_currency_id(recurring_invoice['currency_id'])
        recurring_invoice_obj.set_price_precision(\
        recurring_invoice['price_precision'])
        recurring_invoice_obj.set_currency_code(\
        recurring_invoice['currency_code'])
        recurring_invoice_obj.set_currency_symbol(\
        recurring_invoice['currency_symbol'])
        recurring_invoice_obj.set_exchange_rate(\
        recurring_invoice['exchange_rate'])
        recurring_invoice_obj.set_discount(recurring_invoice['discount'])
        recurring_invoice_obj.set_is_discount_before_tax(\
        recurring_invoice['is_discount_before_tax'])
        recurring_invoice_obj.set_discount_type(\
        recurring_invoice['discount_type'])
        line_items = []
        for value in recurring_invoice['line_items']:
            line_item = LineItem()
            line_item.set_item_id(value['item_id'])
            line_item.set_line_item_id(value['line_item_id'])
            line_item.set_name(value['name'])
            line_item.set_description(value['description'])
            line_item.set_item_order(value['item_order'])
            line_item.set_rate(value['rate'])
            line_item.set_quantity(value['quantity'])
            line_item.set_unit(value['unit'])
            line_item.set_discount_amount(value['discount_amount'])
            line_item.set_discount(value['discount'])
            line_item.set_tax_id(value['tax_id'])
            line_item.set_tax_name(value['tax_name'])
            line_item.set_tax_type(value['tax_type'])
            line_item.set_tax_percentage(value['tax_percentage'])
            line_item.set_item_total(value['item_total'])
            line_items.append(line_item)
        recurring_invoice_obj.set_line_items(line_items)
        recurring_invoice_obj.set_shipping_charge(\
        recurring_invoice['shipping_charge'])
        recurring_invoice_obj.set_adjustment(recurring_invoice['adjustment'])
        recurring_invoice_obj.set_adjustment_description(\
        recurring_invoice['adjustment_description'])
        recurring_invoice_obj.set_late_fee(recurring_invoice['late_fee'])
        recurring_invoice_obj.set_sub_total(recurring_invoice['sub_total'])
        recurring_invoice_obj.set_total(recurring_invoice['total'])
        recurring_invoice_obj.set_tax_total(recurring_invoice['tax_total'])
        recurring_invoice_obj.set_allow_partial_payments(\
        recurring_invoice['allow_partial_payments'])
        taxes = []
        for value in recurring_invoice['taxes']:
            tax = Tax()
            tax.set_tax_name(value['tax_name'])
            tax.set_tax_amount['value_tax_amount']
            taxes.append(tax)
        recurring_invoice_obj.set_taxes(taxes)
        custom_fields = []
        for value in recurring_invoice['custom_fields']:
            custom_field = CustomField()
            custom_field.set_index(value['index'])
            custom_field.set_value(value['value'])
            custom_fields.append(custom_field)
        recurring_invoice_obj.set_custom_fields(custom_fields)
        payment_gateways = recurring_invoice['payment_options'][\
                           'payment_gateways']
        for value in payment_gateways:
            payment_gateway = PaymentGateway()
            payment_gateway.set_configured(value['configured'])
            payment_gateway.set_additional_field1(value['additional_field1'])
            payment_gateway.set_gateway_name(value['gateway_name'])
            recurring_invoice_obj.set_payment_options(payment_gateway)
        billing_address = recurring_invoice['billing_address']
        billing_address_obj = Address()
        billing_address_obj.set_address(billing_address['address'])
        billing_address_obj.set_city(billing_address['city'])
        billing_address_obj.set_state(billing_address['state'])
        billing_address_obj.set_zip(billing_address['zip'])
        billing_address_obj.set_country(billing_address['country'])
        billing_address_obj.set_fax(billing_address['fax'])
        recurring_invoice_obj.set_billing_address(billing_address_obj)
        shipping_address = recurring_invoice['shipping_address']
        shipping_address_obj = Address()
        shipping_address_obj.set_address(shipping_address['address'])
        shipping_address_obj.set_city(shipping_address['city'])
        shipping_address_obj.set_state(shipping_address['state'])
        shipping_address_obj.set_zip(shipping_address['zip'])
        shipping_address_obj.set_country(shipping_address['country'])
        shipping_address_obj.set_fax(shipping_address['fax'])
        recurring_invoice_obj.set_shipping_address(shipping_address_obj)
        recurring_invoice_obj.set_template_id(recurring_invoice['template_id'])
        recurring_invoice_obj.set_template_name(\
        recurring_invoice['template_name'])
        recurring_invoice_obj.set_notes(recurring_invoice['notes'])
        recurring_invoice_obj.set_terms(recurring_invoice['terms'])
        recurring_invoice_obj.set_salesperson_id(\
        recurring_invoice['salesperson_id'])
        recurring_invoice_obj.set_salesperson_name(\
        recurring_invoice['salesperson_name'])
        recurring_invoice_obj.set_created_time(\
        recurring_invoice['created_time'])
        recurring_invoice_obj.set_last_modified_time(\
        recurring_invoice['last_modified_time'])

        return recurring_invoice_obj

    def get_message(self, response):
        """This method parses the json object and returns the message string.

        Args:
            response(dict): Response containing json object.
 
        Returns:
            string: Success message.

        """
        return response['message']

    def recurring_invoice_history_list(self, response):
        """This method parses the json object and returns list of comments 
            object.

        Args: 
            response(dict): Response containing json object.
 
        Returns:
            instance: Comments list object.

        """
        comments = CommentList()
        for value in response['comments']:
            comment = Comment()
            comment.set_comment_id(value['comment_id'])
            comment.set_recurring_invoice_id(value['recurring_invoice_id'])
            comment.set_description(value['description'])
            comment.set_commented_by_id(value['commented_by_id'])
            comment.set_commented_by(value['commented_by'])
            comment.set_comment_type(value['comment_type'])
            comment.set_date(value['date'])
            comment.set_date_description(value['date_description'])
            comment.set_time(value['time'])
            comment.set_operation_type(value['operation_type'])
            comment.set_transaction_id(value['transaction_id'])
            comment.set_transaction_type(value['transaction_type'])
            comments.set_comments(comment)
        return comments
  
     



#$Id$

class ManualReminderList:
    """This class is used to create object for Reminder list."""
    def __init__(self):
        """Initialize parameters for Reminder list."""
        self.manual_reminders = []

    def set_manual_reminders(self, reminder):
        """Set reminder.

        Args:
            reminder(instance): Reminder object.

        """
        self.manual_reminders.append(reminder)

    def get_manual_reminders(self):
        """Get reminders.

        Returns:
            list of instance: List of manual reminders object.

        """
        return self.manual_reminders



#$Id$

from books.model.PageContext import PageContext

class VendorPaymentList: 
    """This class is used to create object for vendor payments list."""
    def __init__(self):
        """Initialize the parameters for Vendor payments list."""
        self.vendor_payments = []
        self.page_context = PageContext()

    def set_vendor_payments(self, vendor_payments):
        """Set vendor payments.

        Args: 
            vendor_payments(instance): Vendor payments object.

        """
        self.vendor_payments.append(vendor_payments)

    def get_vendor_payments(self): 
        """Get vendor payments.

        Returns:
            list of instance: List of vendor payments object.
       
        """
        return self.vendor_payments

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context

 



#$Id$

class InvoiceCredited:
    """This class is used to create objecctfor Invoices Credited."""
    def __init__(self): 
        """Initialize parameters for Invoices credited."""
        self.creditnote_id = ''
        self.invoice_id = ''
        self.creditnotes_invoice_id = ''
        self.date = ''
        self.invoice_number = ''
        self.creditnotes_number = ''
        self.credited_amount = 0.0
        self.credited_date = ''
        self.amount_applied = 0.0

    def set_creditnote_id(self, creditnote_id):
        """Set credit note id.
      
        Args:
            creditnote_id(str): Credit note id.

        """
        self.creditnote_id = creditnote_id

    def get_creditnote_id(self):
        """Get credit note id.

        Returns:
            str: Credit note id.

        """
        return self.creditnote_id
  
    def set_invoice_id(self, invoice_id):
        """Set invoice id.

        Args:
            invoice_id(str): Invoice id.

        """
        self.invoice_id = invoice_id

    def get_invoice_id(self):
        """Get invoice id.

        Returns:
            str: Invoice id.

        """
        return self.invoice_id

    def set_creditnotes_invoice_id(self, creditnotes_invoice_id):
        """Set credit note invoice id.

        Args:
            creditnotes_invoice_id(str): Credditnote invoice id.

        """
        self.creditnotes_invoice_id = creditnotes_invoice_id
 
    def get_creditnotes_invoice_id(self):
        """Get creditnotes invoice id.

        Returns:
            str: Creditnotes invoice id.

        """
        return self.creditnotes_invoice_id

    def set_date(self, date):
        """Set date.

        Args:
            date(str): Date.

        """
        self.date = date
 
    def get_date(self):
        """Get date.

        Returns:
            str: Date.

        """
        return self.date

    def set_invoice_number(self, invoice_number):
        """Set invoice number.

        Args:
            invoice_number(str): Invoice number.

        """
        self.invoice_number = invoice_number

    def get_invoice_number(self):
        """Get invoice number.
        
        Returns:
            str: Invoice number.

        """
        return self.invoice_number
 
    def set_creditnotes_number(self, creditnotes_number):
        """Set creditnote number.

        Args:
            creditnote_number(str): Creditnote number.

        """
        self.creditnotes_number = creditnotes_number
 
    def get_creditnotes_number(self):
        """Get creditnote number.

        Returns:
            str: Creditnote number.

        """
        return self.creditnotes_number

    def set_credited_amount(self, credited_amount):
        """Set credited amount.

        Args:
            credited_amount(float): Credited amount.

        """
        self.credited_amount = credited_amount

    def get_credited_amount(self):
        """Get credited amount.

        Returns:
            float: Credited amount.
 
        """
        return self.credited_amount

    def set_credited_date(self, credited_date):
        """Set credited date.

        Args:
            credited_date(str): Credited date.

        """
        self.credited_date = credited_date

    def get_credited_date(self):
        """Get credited date.

        Returns: 
            str: Credited date.

        """
        return self.credited_date

    def set_amount_applied(self, amount_applied):
        """Set amount applied.

        Args:
            amount_applied(float): Amount applied.

        """
        self.amount_applied = amount_applied

    def get_amount_applied(self):
        """Get amount applied.

        Returns:
            float: Amount applied.

        """
        return self.amount_applied

   



#$Id$

class TimeEntry: 
    """This class is used to create object for Time Entry."""
    def __init__(self):
        """Initialize parameters for Time entry object."""
        self.time_entry_id = ''
        self.project_id = ''
        self.project_name = ''
        self.task_id = ''
        self.task_name = '' 
        self.user_id = ''
        self.user_name = ''
        self.is_current_user = None
        self.log_date = ''
        self.begin_time = ''
        self.end_time = ''
        self.log_time = ''
        self.billed_status = ''
        self.notes = ''
        self.time_started_at = ''
        self.timer_duration_in_minutes = 0
        self.created_time = ''
        self.customer_id = ''
        self.customer_name = ''
        self.start_timer = None

    def set_start_timer(self, start_timer):
        """Set start timer.

        Args:
            start_timer(bool): Start timer.

        """
        self.start_timer = start_timer

    def get_start_timer(self):
        """Get start timer.

        Returns:
            bool: Start timer.

        """
        return self.start_timer
     
    def set_time_entry_id(self, time_entry_id):
        """Set time entry id.

        Args:
            time_entry_id(str): Time entry id.

        """
        self.time_entry_id = time_entry_id

    def get_time_entry_id(self):
        """Get time entry id.

        Returns:
            str: Time entry id.

        """
        return self.time_entry_id

    def set_project_id(self, project_id):
        """Set project id.

        Args:
            project_id(str): Project id.

        """
        self.project_id = project_id

    def get_project_id(self):
        """Get project id.

        Returns:
            str: Project id.

        """
        return self.project_id

    def set_project_name(self, project_name):
        """Set project name.

        Args:
            project_name(str):Project name.

        """
        self.project_name = project_name

    def get_project_name(self):
        """Get project name.

        Returns:  
            str: Project name.

        """
        return self.project_name

    def set_task_id(self, task_id):
        """Set task id.

        Args: 
            task_id(str): Task id.

        """
        self.task_id = task_id

    def get_task_id(self):
        """Get task id.

        Returns:
            str: Task id.

        """
        return self.task_id

    def set_task_name(self, task_name):
        """Set task name.

        Args:
            task_name(str): Task name.

        """
        self.task_name = task_name

    def get_task_name(self):
        """Get task name.

        Returns:
            str: Task name.

        """
        return self.task_name

    def set_user_id(self, user_id):
        """Set user id.

        Args:
            user_id(str): User id.
 
        """ 
        self.user_id = user_id

    def get_user_id(self):
        """Get user id.

        Returns:
            str: User id.

        """
        return self.user_id

    def set_user_name(self, user_name):
        """Set user name.

        Args:
            user_name(str): User name.

        """
        self.user_name = user_name

    def get_user_name(self):
        """Get user name.

        Returns:
            str: User name.
 
        """
        return self.user_name

    def set_is_current_user(self, is_current_user): 
        """Set whether it is current user or not.

        Args:
            is_current_user(bool): True if it is a current user else false.

        """
        self.is_current_user = is_current_user

    def get_is_current_user(self):
        """Get whether it is current user or not.

        Returns:
            bool: True if it is current user else False.

        """
        return self.is_current_user

    def set_log_date(self, log_date):
        """Set log date.

        Args:
            log_date(str): Log date.

        """
        self.log_date = log_date

    def get_log_date(self):
        """Get log date.

        Returns:
            str: Log date.

        """
        return self.log_date

    def set_begin_time(self, begin_time):
        """Set begin time.

        Args:
            begin_time(str): Begin time.

        """
        self.begin_time = begin_time

    def get_begin_time(self):
        """Get begin time.
 
        Returns:
            str: Begin time.

        """
        return self.begin_time

    def set_end_time(self, end_time):
        """Set end time.

        Args:
            end_time(str): End time.

        """
        self.end_time = end_time

    def get_end_time(self):
        """Get end time.

        Returns:
            str: End time

        """
        return self.end_time

    def set_log_time(self, log_time):
        """Set log time.

        Args:
            log_time(str): Log time.
 
        """
        self.log_time = log_time

    def get_log_time(self):
        """Get log time.

        Returns:
            str: Log time.

        """
        return self.log_time

    def set_billed_status(self, billed_status):
        """Set billed status.

        Args:
            billed_status(str): Billed status.

        """
        self.billed_status = billed_status

    def get_billed_status(self):
        """Get billed status.

        Returns:
            str: Billed status.

        """
        return self.billed_status

    def set_notes(self, notes):
        """Set notes.

        Args:
            notes(str): Notes.

        """
        self.notes = notes

    def get_notes(self):
        """Get notes.

        Returns:
            str: Notes.

        """
        return self.notes

    def set_timer_started_at(self, timer_started_at):
        """Set timer started at.

        Args:
            timer_started_at(str): Timer started at.

        """
        self.timer_started_at = timer_started_at

    def get_timer_started_at(self):
        """Get timer started at.

        Returns:
            str: Timer started at.

        """
        return self.timer_started_at

    def set_timer_duration_in_minutes(self, timer_duration_in_minutes):
        """Set timer duration in minutes.

        Args:
            timer_duration_in_minutes(int): Timer duration in minutes.

        """
        self.timer_duration_in_minutes = timer_duration_in_minutes

    def get_timer_duration_in_minutes(self):
        """Get timer duration in minutes.

        Returns:
            str: Timer duration in minutes.

        """
        return self.timer_duration_in_minutes

    def set_created_time(self, created_time):
        """Set created time.

        Args:
            created_time(str): Created time.

        """
        self.created_time = created_time

    def get_created_time(self):
        """Get created time.

        Returns:
            str: Created time.

        """
        return self.created_time

    def set_customer_id(self, customer_id):
        """Set customer id.

        Args:
            customer_id(str): Customer id.
 
        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get customer id.

        Returns:
            str: Customer id.
 
        """
        return self.customer_id

    def set_customer_name(self, customer_name):
        """Set customer name.
  
        Args:
            customer_name(str): Customer name.
 
        """ 
        self.customer_name = customer_name

    def get_customer_name(self):
        """Get customer name.

        Returns: 
            str: Customer name.
 
        """
        return self.customer_name

    def to_json(self):
         """This method is used to convert time entry object to json format.

         Returns:
             dict: Dictionary containing json object for time entry.

         """
         data = {}
         if self.project_id != '':
             data['project_id'] = self.project_id
         if self.task_id != '':
             data['task_id'] = self.task_id
         if self.user_id != '':
             data['user_id'] = self.user_id
         if self.log_date != '': 
             data['log_date'] = self.log_date
         if self.begin_time != '':
             data['begin_time'] = self.begin_time
         if self.end_time != '':
             data['end_time'] = self.end_time
         if self.log_time != '':
             data['log_time'] = self.log_time
         if self.notes != '':
             data['notes'] = self.notes
         if self.start_timer != '':
             data['start_timer'] = self.start_timer
         return data



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.ExpensesParser import ExpensesParser
from os.path import basename
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'expenses/'
parser = ExpensesParser()
zoho_http_client = ZohoHttpClient()

class ExpensesApi:
    """Expenses Api is used to:
    
    1.List expenses with pagination.
    2.Get the details of an expense.
    3.Create a billable or non-billable expense.
    4.Update an existing expense.
    5.Delete an expense.
    6.Get history and comments of an expense.
    7.Returns the receipt attached to an expense.
    8.Attach a receipt to an expense.
    9.Delete the receipt attached to the expense.

    """
    def __init__(self, authtoken, organization_id):
        """Initialize Expenses Api using user's authtoken and organization 
            id.
 
        Args: 
            authtoken(str): User's Authtoken.
            organization_id(str): User's organization id.

        """
        self.details = {
            'authtoken':authtoken, 
            'organization_id':organization_id
            }
  
    def get_expenses(self, parameter=None): 
        """List expenses with pagination.
 
        Args:
            parameter(dict, optional): Filter with which expenses list has to 
                be displayed. Defaults to None.

        Returns:
            instance: Expenses list object.

        """
        resp = zoho_http_client.get(base_url, self.details, parameter)
        return parser.get_list(resp) 
  
    def get(self, expense_id):
        """Get details of an expense.

        Args:
            expense_id(str): Expense id.

        Returns:
            instance: Expense object.

        """
        url = base_url + expense_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_expense(resp) 

    def create(self, expense, receipt=None):
        """Create an expense.

        Args:
            expense(instance): Expense object.
            receipt(file, optional): Expense receipt file to attach.Allowed 
                Extensions: gif, png, jpeg, jpg, bmp and pdf.

        Returns:
            instance: Expense object.

        """
        json_object = dumps(expense.to_json())
        data = {
            'JSONString': json_object
            }
        if receipt is not None:
            attachments = [{
                'receipt': {
                    'filename': basename(receipt),
                    'content': open(receipt).read()
                    }
                }]
        else:
            attachments = None
        resp = zoho_http_client.post(base_url, self.details, data, None, \
                                     attachments)
        return parser.get_expense(resp) 

    def update(self, expense_id, expense, receipt=None):
        """Update an existing expense.

        Args:
            expense_id(str): Expense id.
            expense(instance): Expense object.
            receipt(file): Expense receipt file to attach. Allowed Extensions: 
                gif, png, jpeg, jpg, bmp and pdf.

        Returns:
            instance: Expense object.

        """
        url = base_url + expense_id
        json_object = dumps(expense.to_json())
        data = {
            'JSONString': json_object
            }
        if receipt is None:
            attachments = None
        else:
            attachments = [{
                'receipt': {
                    'filename': basename(receipt),
                    'content': open(receipt).read()
                    }
                }]
        resp = zoho_http_client.put(url, self.details, data, None,
                                     attachments)
        return parser.get_expense(resp) 
  
    def delete(self, expense_id):
        """Delete an existing expense.

        Args:
            expense_id(str): Expense id.

        Returns:
            str: Success message('The expense has been deleted.').

        """
        url = base_url + expense_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
  
    def list_comments_history(self, expense_id):
        """Get history and comments of an expense.

        Args:
            expense_id(str): Expense id.

        Returns:
            instance: comments list object.
 
        """
        url = base_url + expense_id + '/comments'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_comments(resp) 

    def get_receipt(self, expense_id, preview=None):
        """Get the receipt attached to an expense.

        Args:
            expense_id(str): Expense id.
            preview(bool, optional): True to get the thumbnail of the receipt.

        Returns:
            file: Returns the receipt attached to the expense.
 
        """
        url = base_url + expense_id + '/receipt'
        if preview is not None:
            query = {
                'preview': preview
                }
        else:
            query = None
        resp = zoho_http_client.getfile(url, self.details, query)
        return resp 

    def add_receipt(self, expense_id, receipt):
        """Attach a receipt to an expense.

        Args:
            expense_id(str): Expense id.
            receipt(file): Receipt to be attached.Allowed Extensions: gif, png,
             jpeg, jpg, bmp, pdf, xls, xlsx, doc and docx.

        Returns:
            str: Success message('The expense receipt has been attached.').

        """
        url = base_url + expense_id + '/receipt'
        attachments = [{
                'receipt': {
                    'filename': basename(receipt),
                    'content': open(receipt).read()
                    }
                }]
        data = { 
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data, None, attachments)
        return parser.get_message(resp) 

    def delete_receipt(self, expense_id):
        """Delete the receipt attached to the expense.
 
        Args:
            expense_id(str): Expense id.

        Returns:
            str: Success message('The attached expense receipt has been 
                deleted.').

        """
        url = base_url + expense_id + '/receipt'
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
  



#$Id$

class Currency:
    """This class is used to create object for currency. """
    def __init__(self):
        """Initialize parameters for currency object."""
        self.currency_id = ''
        self.currency_code = ''
        self.currency_name = ''
        self.currency_symbol = ''
        self.price_precision = 0
        self.currency_format = ''
        self.is_base_currency = None
        self.exchange_rate = 0.0
        self.effective_date = ''

    def set_currency_id(self, currency_id):
        """Set currency id.
 
        Args:
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self):
        """Get currency id.

        Returns:
            str: Currency id.

        """ 
        return self.currency_id

    def set_currency_code(self, currency_code):
        """Set currency code.
 
        Args:
            currency_code(str): Currency code.

        """
        self.currency_code = currency_code

    def get_currency_code(self):
        """Get currency code.

        Returns:
            str: Currency code.

        """
        return self.currency_code

    def set_currency_name(self, currency_name):
        """Set currency name.

        Args:
            currency_name(str): Currency name.

        """
        self.currency_name = currency_name

    def get_currency_name(self): 
        """Get currency name.

        Returns:
            str: Currency name.

        """
        return self.currency_name

    def set_currency_symbol(self, currency_symbol):
        """Set currency symbol.

        Args:
            currency_symbol(str): Currency symbol.

        """
        self.currency_symbol = currency_symbol

    def get_currency_symbol(self):
        """Get currency symbol.

        Returns:
            str: Currency symbol.

        """
        return self.currency_symbol

    def set_price_precision(self, price_precision):
        """Set price precision.

        Args:
            price_precision(int): Price precision.

        """
        self.price_precision = price_precision

    def get_price_precision(self):
        """Get price precision.

        Returns:
            int: Price precision.
 
        """
        return self.price_precision

    def set_currency_format(self, currency_format):
        """Set currency format.

        Args:
            currency_format(str): Currency format.

        """
        self.currency_format = currency_format

    def get_currency_format(self):
        """Get currency format.

        Returns:
            str: Currency fromat.

        """
        return self.currency_format

    def set_is_base_currency(self, is_base_currency):
        """Set whether the currency is base currency.

        Args:
            is_base_currency(bool): True if it is base currency else False.
      
        """
        self.is_base_currency = is_base_currency

    def get_is_base_currency(self):
        """Get whether the currency is base currency.

        Returns:
            bool: True if it is base currency else false.

        """
        return self.is_base_currency

    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args:
            exchange_rate(float): Exchange rate.

        """
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self):
        """Get exchange rate.

        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate

    def set_effective_date(self, effective_date):
        """Set effective date.

        Args:
            effective_date(str): Effective date.

        """
        self.effective_date = effective_date

    def get_effective_date(self):
        """Get effective date.

        Returns:
            str: Effective date.

        """
        return self.effective_date

    def to_json(self):
        """This method is used to create json object for currency.

        Returns:
            dict: Dictionary containing json object for currency.

        """
        data = {}
        if self.currency_code != '':
            data['currency_code'] = self.currency_code
        if self.currency_symbol != '':
            data['currency_symbol'] = self.currency_symbol
        if self.price_precision > 0:
            data['price_precision'] = self.price_precision
        if self.currency_format != '':
            data['currency_format'] = self.currency_format
        return data



#$Id$#

class BankRuleList:
    """This class is used to create object for Bank rules list."""
    def __init__(self):
        """Initialize parameters for bank rules list."""
 
        self.bank_rules = []

    def set_bank_rules(self, bank_rules):
        """Set bank rules.
 
        Args: 
            bank_rules(instance): Bank rules object.

        """
        self.bank_rules.append(bank_rules)

    def get_bank_rules(self):
        """Get bank rules.

        Returns: 
            list of instance: List of bank rules object.

        """
        return self.bank_rules 



#$Id$

from books.model.PageContext import PageContext

class CurrencyList:
    """This class is used to create object for currency list."""
    def __init__(self):
        """Initialize parameters for currency list."""
        self.currencies = []
        self.page_context = PageContext()

    def set_currencies(self, currency):
        """Set currencies.

        Args:
            currency(instance): Currency object.

        """
        self.currencies.append(currency)
 
    def get_currencies(self):
        """Get currencies.

        Returns:
            list of instance: List of currency object.

        """
        return self.currencies

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context





#$Id$

from books.model.PageContext import PageContext

class EstimateList:
    """This class is used to create object for estimates list."""
    def __init__(self):
        """Initialize parameters for estimates list."""
        self.estimates = []
        self.page_context = PageContext()

    def set_estimates(self, estimate):
        """Set estimates.

        Args:
            estimate(instance): Estimate object.
 
        """
        self.estimates.append(estimate)

    def get_estimates(self):
        """Get estimates.
       
        Returns:
            list of estimetes object.

        """
        return self.estimates

    def set_page_context(self, page_context):
        """Set page context.
    
        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context




#$Id$

from books.model.Preference import Preference
from books.model.CustomField import CustomField
from books.model.PlaceholderAddressFormat import PlaceholderAddressFormat

class PreferenceList:
    """This class is used to create object for Preferences list."""
    def __init__(self): 
        """Initialize parameters for preferences list."""
        self.preferences = Preference()
        self.custom_fields = CustomField()
        self.placeholders_address_format = PlaceholderAddressFormat()

    def set_preferences(self, preference):
        """Set preferences.

        Args:
            preference(instance): Preference object.

        """
        self.preferences = preference

    def get_preferences(self):
        """Get preferneces.

        Retruns:
            instance: Preferences object.

        """
        return self.preferences

    def set_custom_fields(self, custom_fields):
        """Set custom fields.

        Args:
            custom_fields(instance): Custom fields.

        """
        self.custom_fields = custom_fields

    def set_placeholders_address_format(self, placeholders_address_format):
        """Set placeholders address format.

        Args:
            placeholders_address_format(instance): Place holders address format.

        """
        self.placeholdedrs_address_format = placeholders_address_format

    def get_placeholders_address_format(self):
        """Get placeholders address format.

        Returns:
            instance: Placeholders address format.

        """
        return self.placeholders_address_format




#$Id$

class PageContext:
    """This class is used to create object for page context."""
    def __init__(self):
        """Initialize parameters for page context."""
        self.page = 0
        self.per_page = 0
        self.has_more_page = None
        self.applied_filter = ''
        self.sort_column = ''
        self.sort_order = ''
        self.report_name = ''
        self.search_criteria = []
        self.from_date = ''
        self.to_date = ''

    def set_page(self, page):
        """Set page number for the list.
     
        Args:
            page(int): Page number for list.
 
        """
        self.page = page

    def get_page(self):
        """Get page number for the list.
       
        Returns:
            int: Page number for the list.
 
        """
        return self.page
 
    def set_per_page(self, per_page):
        """Set details per page.
 
        Args:
            per_page(int): Details per page.

        """
        self.per_page = per_page

    def get_per_page(self):
        """Get details per page.
 
        Returns:
            int: Details per page.

        """ 
        return self.per_page

    def set_has_more_page(self, has_more_page):
        """Set has more page.
    
        Args:
            has_more_page(bool): Has more page.

        """
        self.has_more_page = has_more_page

    def get_has_more_page(self):
        """Get has more page.
 
        Returns:
            bool: Has more page.

        """
        return self.has_more_page
  
    def set_applied_filter(self, applied_filter):
        """Set applied filter for the report.
 
        Args:
            applied_filter(str): Applied filter for the report.

        """
        self.applied_filter = applied_filter

    def get_applied_filter(self):
        """Get applied filter for the report.
     
        Returns:
            str: Applied filter for the report.
  
        """
        return self.applied_filter

    def set_sort_column(self, sort_column):
        """Set sort column.
      
        Args: 
            sort_column(str): Sort column.
 
        """
        self.sort_column = sort_column

    def get_sort_column(self):
        """Get sort column.
          
        Returns: 
            str: Sort column.

        """
        return self.sort_column

    def set_sort_order(self, sort_order):
        """Set sort order.
          
        Args:
            sort_order(str): Sort order.
       
        """
        self.sort_order = sort_order

    def get_sort_order(self):
        """Get sort order.
         
        Returns:
            str: Sort order.
  
        """
        return self.sort_order

    def set_report_name(self, report_name):
        """Set report name.
       
        Args: 
            report_name(str): Report name.
     
        """
        self.report_name = report_name

    def get_report_name(self):
        """Get report name.
 
        Returns:
            str: Report name.

        """
        return self.report_name

    def set_search_criteria(self, search_criteria):
        """Set search criteria.

        Args:
            search_criteria(instance): Search criteria object.

        """
        self.search_criteria.append(search_criteria)

    def get_search_criteria(self):
        """Get search criteria.

        Returns:
            list of instance: List of search criteria object.

        """
        return self.search_criteria

    def set_from_date(self, from_date):
        """Set from date.

        Args:
            from_date(str): From date.

        """
        self.from_date = from_date

    def get_from_date(self):
        """Get from date.

        Returns:
            str: From date.

        """
        return self.from_date

    def set_to_date(self, to_date):
        """Set to date.

        Args:
            to_date(str): To date.

        """
        self.to_date = to_date

    def get_to_date(self):
        """Get to date.

        Returns: 
            to_date(str): To date.

        """
        return self.to_date
 
   



#$Id$

class BankRule:
    """This class is used to create object for bank rules."""
    def __init__(self):
        """Initialize parameters for Bank rules."""
        self.rule_id = ''
        self.rule_name = ''
        self.rule_order = 0
        self.apply_to = ''
        self.target_account_id = ''
        self.apply_to = ''
        self.criteria_type = ''
        self.criterion = []
        self.record_as = ''
        self.account_id = ''
        self.account_name = ''
        self.tax_id = ''
        self.reference_number = ''
        self.customer_id = ''
        self.customer_name = ''

    def set_rule_id(self, rule_id): 
        """Set rule id.

        Args:
            rule_id(str): Rule id.

        """
        self.rule_id = rule_id

    def get_rule_id(self): 
        """Get rule id.
           
        Returns:
            str: Rule id.

        """
        return self.rule_id

    def set_rule_name(self, rule_name):
        """Set rule name.

        Args:
            rule_name(str): Rule name.

        """
        self.rule_name = rule_name

    def get_rule_name(self):
        """Get rule name.

        Returns:
            str:Rule name.

        """
        return self.rule_name

    def set_rule_order(self, rule_order):
        """Set rule order.

        Args:
            rule_order(int): Rule order.

        """
        self.rule_order = rule_order

    def get_rule_order(self):
        """Get rule order.

        Returns:
            int: Rule order.

        """
        return self.rule_order

    def set_apply_to(self, apply_to):
        """Set apply to.

        Args:
            apply_to(str): Apply to.

        """
        self.apply_to = apply_to

    def get_apply_to(self):
        """Get apply to.

        Returns:
            str: Apply to.

        """
        return self.apply_to

    def set_criteria_type(self, criteria_type):
        """Set criteria type.

        Args:
            criteria_type(str): Criteria type.

        """
        self.criteria_type = criteria_type

    def get_criteria_type(self):
        """Get criteria type.

        Returns: 
            str: Criteria type.

        """
        return self.criteria_type

    def set_target_account_id(self, target_account_id):
        """Set target account id.

        Args:
            target_account_id(str): Target account id.

        """
        self.target_account_id = target_account_id

    def get_target_account_id(self):
        """Get target account id.

        Returns:
            str: Target account id.

        """
        return self.target_account_id
  
    def set_criterion(self, criteria):
        """Set criterion.

        Args:
            criteria(instance): Criteria object.

        """
        self.criterion.append(criteria)

    def get_criterion(self): 
        """Get criterion.

        Returns: 
            list of instance: List of criteria object.

        """
        return self.criterion

    def set_record_as(self, record_as):  
        """Set record as.

        Args:
            record_as(str): Record as.

        """
        self.record_as = record_as

    def get_record_as(self):
        """Get record as.

        Returns:
            str: Record as.

        """
        return self.record_as

    def set_account_id(self, account_id):
        """Set account id.

        Args:
            account_id(str): Account id.

        """
        self.account_id = account_id

    def get_account_id(self):
        """Get account id.

        Returns:
            str: Account id.

        """
        return self.account_id

    def set_account_name(self, account_name):
        """Set account name.

        Args:
            account_name(str): Account name.

        """
        self.account_name = account_name

    def get_account_name(self):
        """Get account name.

        Returns:
            str: Account name.

        """
        return self.account_name

    def set_tax_id(self, tax_id):
        """Set tax id.
  
        Args:
            tax_id(str): Tax id.

        """	
        self.tax_id = tax_id

    def get_tax_id(self):
        """Get tax id.

        Returns:
            str:Tax id.

        """
        return self.tax_id

    def set_reference_number(self, reference_number): 
        """Set reference number.

        Args: 
            reference_number(str): Reference number.

        """
        self.reference_number = reference_number

    def get_reference_number(self):
        """Get reference number.

        Returns:
            str: Reference number.

        """
        return self.reference_number

    def set_customer_id(self, customer_id):
        """Set customer id.

        Args: 
            customer_id(str): Customer id.

        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get customer id.

        Returns: 
            str: Customer id.

        """
        return self.customer_id

    def set_customer_name(self, customer_name):
        """Set customer name.

        Args:
            customer_name(str): Customer name.

        """
        self.customer_name = ''
 
    def get_customer_name(self):
        """Get customer name.

        Returns:
            str: Customer name.
 
        """
        return self.customer_name

    def to_json(self):
        """This method is used to convert bills object to json object.

        Returns:
            dict: Dictionary containing json object for bank rules.

        """
        data = {}
        if self.rule_name != '':
            data['rule_name'] = self.rule_name
        if self.target_account_id != '':
            data['target_account_id'] = self.target_account_id
        if self.apply_to != '':
            data['apply_to'] = self.apply_to
        if self.criteria_type != '':
            data['criteria_type'] = self.criteria_type
        if self.criterion:
            data['criterion'] = []
            for value in self.criterion:
               criteria = value.to_json()
               data['criterion'].append(criteria)
        if self.record_as != '':
            data['record_as'] = self.record_as
        if self.account_id != '':
            data['account_id'] = self.account_id
        if self.tax_id != '':
            data['tax_id'] = self.tax_id
        if self.reference_number != '':
            data['reference_number'] = self.reference_number
        if self.customer_id != '':
            data['customer_id'] = self.customer_id
        return data



#$Id$

class Instrumentation:
    """This class is used tocreate object for instrumentation."""
    def __init__(self):
        """Initialize parameters for Instrumentation object."""
        self.query_execution_time = ''
        self.request_handling_time = ''
        self.response_write_time = ''
        self.page_context_write_time = ''

    def set_query_execution_time(self, query_execution_time):
        """Set query execution time.

        Args:
            query_execution_time(str): Query execution time.
            
        """
        self.query_execution_time = query_execution_time

    def get_query_execution_time(self):
        """Get query execution time.

        Returns:
            str: Query execution time.

        """
        return self.query_execution_time

    def set_request_handling_time(self, request_handling_time): 
        """Set request handling time.
 
        Args:
            request_handling_time(str): Request handling time.

        """
        self.request_handling_time = request_handling_time

    def get_request_handling_time(self): 
        """Get request handling time.

        Returns:
            str: Request handling time.

        """
        return self.request_handling_time

    def set_response_write_time(self, response_write_time):
        """Set response write time.

        Args: 
            response_write_time(str): Response write time.

        """
        self.response_write_time = response_write_time
 
    def get_response_write_time(self):
        """Get response write time.

        Returns:
            str: Response write time.

        """
        return self.response_write_time

    def set_page_context_write_time(self, page_context_write_time):
        """Set page context write time.

        Args: 
            page_context_write_time(str): Page context write time.

        """
        self.page_context_write_time = page_context_write_time

    def get_page_context_write_time(self): 
        """Get page context write time.

        Returns:
            str: Page context write time.

        """
        return self.page_context_write_time




#$Id$

class DefaultTemplate:
    """This class is used to create object for default templates."""
    def __init__(self):
        """Initialize the parameters for default temmplates."""
        self.invoice_template_id = ''
        self.invoice_template_name = ''
        self.estimate_template_id = ''
        self.estimate_template_name = ''
        self.creditnote_template_id = ''
        self.creditnote_template_name = ''
        self.invoice_email_template_id = ''
        self.invoice_email_template_name = ''
        self.estimate_email_template_id = ''
        self.estimate_email_template_name = ''
        self.creditnote_email_template_id = ''
        self.creditnote_email_template_name = ''
                               
    def set_invoice_template_id(self, invoice_template_id):
        """Set default invoice template id used for this contact while 
            creating invoice.
    
        Args:
            invoice_template_id(str): Default invoice template id used for this
                contact while creating invoice.

        """
        self.invoice_template_id = invoice_template_id

    def get_invoice_template_id(self):
        """Get default invoice template id used for this contact while 
            creating invoice.
          
        Returns:
            str: Default invoice template id used for this contact while 
                creating invoice.
 
        """
        return self.invoice_template_id

    def set_invoice_template_name(self, invoice_template_name):
        """Set default invoice template name used for this contact while 
            creating invoice.
         
        Args:
            invoice_template_name(str): Default invoice template name used for 
                this contact while creating invoice.
  
        """
        self.invoice_template_name = invoice_template_name

    def get_invoice_template_name(self):
        """Get default invoice template name used for this contact while 
            creating invoice.
  
        Returns:
            str: Default invoice template name used for this contact while 
                creating invoice.
 
        """
        return self.invoice_template_name
  
    def set_estimate_template_id(self, estimate_template_id):
        """Set default estimate template id used for this contact while creating 
            estimate.
        
        Args:
            estimate_template_id(str): Default estimate template id used for  
                this contact while creating estimate.
 
        """
        self.estimate_template_id = estimate_template_id

    def get_estimate_template_id(self):
        """Get default estimate template id used for this contact while 
            creating estimate.
 
        Returns:
            str: Default estimate template id used for this contact while 
                creating estimate.
 
        """
        return self.estimate_template_id
  
    def set_estimate_template_name(self, estimate_template_name):
        """Set default estimate template name used for this contact while 
            creating estimate.
        
        Args: 
            estimate_template_name(str): Default estimate template name used 
                for this contact while creating estimate.

        """     
        self.estimate_template_name = estimate_template_name

    def get_estimate_template_name(self):
        """Get default estimate template name used for this contact while 
            creating estimate.
 
        Returns:
            str: Default estimate template name used for this contact while 
                creating estimate.
   
        """ 
        return self.estimate_template_name

    def set_creditnote_template_id(self, creditnote_template_id):
        """Set default creditnote template id used for this contact while 
            creating creditnote.
  
        Args: 
            creditnote_template_id(str): Default creditnote template id used 
                for this contact while creating creditnote.

        """ 
        self.creditnote_template_id = creditnote_template_id

    def get_creditnote_template_id(self):
        """Get default creditnote template id used for this contact while 
            creating creditnote.
   
        Returns: 
            str: Default creditnote template id used for this contact while 
                creating creditnote.
 
        """
        return self.creditnote_template_id
  
    def set_creditnote_template_name(self, creditnote_template_name):
        """Set default creditnote template name used for this contact while 
            creating creditnote.
  
        Args: 
            creditnote_template_name(str): Default creditnote template name 
                used for this contact while creating creditnote.

        """ 
        self.creditnote_template_name = creditnote_template_name

    def get_creditnote_template_name(self):
        """Get default creditnote template id used for this contact while 
            creating creditnote.
   
        Returns: 
            str: Default creditnote template id used for this contact while 
                creating creditnote.
 
        """
        return self.creditnote_template_name

    def set_invoice_email_template_id(self, invoice_email_template_id):
        """Set default invoice email template id used for this contact while 
            creating invoice.
        
        Args: 
            invoice_email_template_id(str): Default invoice template id used 
                for this contact while creating invoice

        """
        self.invoice_email_template_id = invoice_email_template_id

    def get_invoice_email_template_id(self):
        """Get default invoice email template id used for this contact while 
            creating invoice.
         
        Returns: 
            str: Default invoice email template id used for this contact while 
                creating invoice.
         
        """
        return self.invoice_email_template_id

    def set_invoice_email_template_name(self, invoice_email_template_name):
        """Set default invoice email template name used for this contact while 
            creating invocie.

        Args: 
            invoice_email_template_name(str): Default invoice email template 
                name used for this contact while creating invocie.

        """ 
        self.invoice_email_template_name = invoice_email_template_name
 
    def get_invoice_email_template_name(self):
        """Get default invoice email template name used for this contact while 
            creating invoice.
 
        Returns: 
            str: Default invoice email template name used for this contact while 
                creating invoice.
 
        """
        return self.invoice_email_template_name

    def set_estimate_email_template_id(self, estimate_email_template_id):
        """Set default estimate template id used for this contact while 
            creating estimate.
 
        Args: 
            estimate_email_template_id(str): Default estimate template id 
                used for this cotnact while creating estimate.
 
        """
        self.estimate_email_template_id = estimate_email_template_id
 
    def get_estimate_email_template_id(self):
        """Get default estimate email template id used for this contact while 
            creating estimate.
 
        Returns:
            str: Default estimate template id used for this cotnact while 
                creating estimate.
 
        """
        return self.estimate_email_template_id

    def set_estimate_email_template_name(self, estimate_email_template_name):
        """Set default estimate email template name used for this contact 
            while creating estimate.
        
        Args:
            estimate_email_template_name(str): Default estimate email template 
                name used for this contact while ccreating estimate.
 
        """
        self.estimate_email_template_name = estimate_email_template_name

    def get_estimate_email_template_name(self):
        """Get default estimate email template name used for this contact while
             creating estimate.
       
        Returns: 
            str: Default estimate email template name used for this contact
                while creating estimate.

        """
        return self.estimate_email_template_name
   
    def set_creditnote_email_template_id(self, creditnote_email_template_id):
        """Set default creditnote email template id used for this contact while
             creating creditnote.
 
        Args:
            creditnote_email_template_id(str): Default creditnote email 
                template id used for this contact while creating creditnote.
 
        """
        self.creditnote_email_template_id = creditnote_email_template_id

    def get_creditnote_email_template_id(self):
        """Get default creditnote email template id used for this contact while 
            creating creditnote.
        
        Returns:
            str: Default creditnote email template id used for this contact 
                while creating creditnote.

        """ 
        return self.creditnote_email_template_id

    def set_creditnote_email_template_name(self, 
                                           creditnote_email_template_name):
        """Set default creditnote email template name used for this contact 
            while creating creditnote.
 
        Args:
            creditnote_email_template_name(str): Default creditnote email 
                template name used for this contact while creating creditnote.
 
        """
        self.creditnote_email_template_name = creditnote_email_template_name

    def get_creditnote_email_template_name(self):
        """Get default creditnote email template name used for this contact 
            while creating creditnote.
        
        Returns:
            str: Default creditnote email template name used for this contact 
                while creating creditnote.

        """ 
        return self.creditnote_email_template_name
  

                        
    




#$Id$#

from books.model.ProjectList import ProjectList
from books.model.Project import Project
from books.model.PageContext import PageContext
from books.model.TaskList import TaskList
from books.model.Task import Task
from books.model.User import User
from books.model.TimeEntryList import TimeEntryList
from books.model.TimeEntry import TimeEntry
from books.model.Comment import Comment
from books.model.InvoiceList import InvoiceList
from books.model.Invoice import Invoice
from books.model.UserList import UserList
from books.model.CommentList import CommentList

class ProjectsParser:
    """This class is used to parse the json object for Projects."""
  
    def get_projects_list(self, resp):
        """This method parses the given response and returns projects list 
            object.

        Args:
            resp(dict): Response containing json object for projects.

        Returns:
            instance: Projects list object.

        """
        projects_list = ProjectList()
        for value in resp['projects']:
            project = Project()
            project.set_project_id(value['project_id'])
            project.set_project_name(value['project_name'])
            project.set_customer_id(value['customer_id'])
            project.set_customer_name(value['customer_name'])
            project.set_description(value['description'])
            project.set_status(value['status'])
            project.set_billing_type(value['billing_type'])
            #project.set_rate(value['rate'])
            project.set_created_time(value['created_time'])
            projects_list.set_projects(project)
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        projects_list.set_page_context(page_context_obj)
        return projects_list

    def get_project(self, resp):
        """This method parses the given response and returns projects object.

        Args:
            resp(dict): Dictionary containing json object for projects.

        Returns:
            instance: Projects object.

        """
        project_obj = Project()
        project = resp['project']
        project_obj.set_project_id(project['project_id'])
        project_obj.set_project_name(project['project_name'])
        project_obj.set_customer_id(project['customer_id'])
        project_obj.set_customer_name(project['customer_name'])
        project_obj.set_currency_code(project['currency_code'])
        project_obj.set_description(project['description'])
        project_obj.set_status(project['status'])
        project_obj.set_billing_type(project['billing_type'])
        project_obj.set_budget_type(project['budget_type'])
        project_obj.set_total_hours(project['total_hours'])
        project_obj.set_billed_hours(project['billed_hours'])
        project_obj.set_un_billed_hours(project['un_billed_hours'])
        project_obj.set_created_time(project['created_time'])
        for value in project['tasks']:
            task = Task()
            task.set_task_id(value['task_id'])
            task.set_task_name(value['task_name'])
            task.set_description(value['description'])
            task.set_rate(value['rate'])
            task.set_budget_hours(value['budget_hours'])
            task.set_total_hours(value['total_hours'])
            task.set_billed_hours(value['billed_hours'])
            task.set_un_billed_hours(value['un_billed_hours'])
            project_obj.set_tasks(task)
        for value in project['users']:
            user = User()
            user.set_user_id(value['user_id'])
            user.set_is_current_user(value['is_current_user'])
            user.set_user_name(value['user_name'])
            user.set_email(value['email'])
            user.set_user_role(value['user_role'])
            user.set_status(value['status'])
            user.set_rate(value['rate'])
            user.set_budget_hours(value['budget_hours'])
            user.set_total_hours(value['total_hours'])
            user.set_billed_hours(value['billed_hours'])
            user.set_un_billed_hours(value['un_billed_hours'])
            project_obj.set_users(user)
        return project_obj

    def get_message(self, resp):
        """This method parses the given response and returns string message.

        Args:
            resp(dict): Response containing json object for string message.

        Returns:
            str: Success message.

        """
        return resp['message']

    def get_tasks_list(self, resp):
        """This method parses the given response and returns tasks list object.

        Args:
            resp(dict): Response containing json object for tasks list.


        Returns:
            instance: Task list object.

        """
        task_list = TaskList()
        for value in resp['task']:
            task = Task()
            task.set_project_id(value['project_id'])
            task.set_task_id(value['task_id'])
            task.set_currency_id(value['currency_id'])  
            task.set_customer_id(value['customer_id'])
            task.set_task_name(value['task_name'])
            task.set_project_name(value['project_name'])
            task.set_customer_name(value['customer_name'])
            task.set_billed_hours(value['billed_hours'])
            task.set_log_time(value['log_time'])
            task.set_un_billed_hours(value['un_billed_hours'])
            task_list.set_tasks(task)     
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        task_list.set_page_context(page_context_obj)
        return task_list

    def get_task(self, resp):
        """This method parses the given response and returns task object.

        Args:
            resp(dict): Response containing json object for task.

        Returns:
            instance: Task object.

        """
        task_obj = Task()
        task = resp['task']
        task_obj.set_project_id(task['project_id'])
        task_obj.set_project_name(task['project_name'])
        task_obj.set_task_id(task['task_id'])
        task_obj.set_task_name(task['task_name'])
        task_obj.set_description(task['description'])
        task_obj.set_rate(task['rate'])
        return task_obj

    def get_users(self, resp):
        """This method parses the given response and returns list of users 
            object.

        Args:
            resp(dict): Response containing json object for users list object.

        Returns: 
            instance: Users list object.

        """
        users = UserList()
        for value in resp['users']:
            user = User()
            user.set_user_id(value['user_id'])
            user.set_is_current_user(value['is_current_user'])
            user.set_user_name(value['user_name'])
            user.set_email(value['email'])
            user.set_user_role(value['user_role'])
            user.set_status(value['status'])
            user.set_rate(value['rate'])
            user.set_budget_hours(value['budget_hours'])
            users.set_users(user)
        return users

    def get_user(self, resp):
        """This method parses the given response and returns user object.

        Args:
            resp(dict): Response containing json object for user.

        Returns:
            instance: User object.

        """
        user_obj = User()
        user = resp['user']
        user_obj.set_project_id(user['project_id'])
        user_obj.set_user_id(user['user_id'])
        user_obj.set_is_current_user(user['is_current_user'])
        user_obj.set_user_name(user['user_name'])
        user_obj.set_email(user['email'])
        user_obj.set_user_role(user['user_role'])
        return user_obj

    def get_time_entries_list(self, resp):
        """This method parses the given response and returns time entries list 
            object.

        Args:
            resp(dict): Response containing json object for time entries.

        Returns:
            instance: Time entries list object.

        """
        time_entries_list = TimeEntryList()
        for value in resp['time_entries']:
            time_entry = TimeEntry()
            time_entry.set_time_entry_id(value['time_entry_id'])
            time_entry.set_project_id(value['project_id'])
            time_entry.set_project_name(value['project_name'])
            time_entry.set_customer_id(value['customer_id'])
            time_entry.set_customer_name(value['customer_name'])
            time_entry.set_task_id(value['task_id'])
            time_entry.set_task_name(value['task_name'])
            time_entry.set_user_id(value['user_id'])
            time_entry.set_is_current_user(value['is_current_user'])
            time_entry.set_user_name(value['user_name'])
            time_entry.set_log_date(value['log_date'])
            time_entry.set_begin_time(value['begin_time'])
            time_entry.set_end_time(value['end_time'])
            time_entry.set_log_time(value['log_time'])
            time_entry.set_billed_status(value['billed_status'])
            time_entry.set_notes(value['notes'])
            time_entry.set_timer_started_at(value['timer_started_at'])
            time_entry.set_timer_duration_in_minutes(value[\
            'timer_duration_in_minutes'])
            time_entry.set_created_time(value['created_time'])
            time_entries_list.set_time_entries(time_entry)
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name']) 
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        time_entries_list.set_page_context(page_context_obj)
        return time_entries_list

    def get_time_entry(self, resp):
        """This method parses the given response and returns time entry object.

        Args:
            resp(dict):Response containing json object for time entry.

        Returns: 
            instance: Time entry object

        """
        time_entry = resp['time_entry']
        time_entry_obj = TimeEntry()
        time_entry_obj.set_time_entry_id(time_entry['time_entry_id'])
        time_entry_obj.set_project_id(time_entry['project_id'])
        time_entry_obj.set_project_name(time_entry['project_name'])
        time_entry_obj.set_task_id(time_entry['task_id'])
        time_entry_obj.set_task_name(time_entry['task_name'])
        time_entry_obj.set_user_id(time_entry['user_id'])
        time_entry_obj.set_user_name(time_entry['user_name'])
        time_entry_obj.set_is_current_user(time_entry['is_current_user'])
        time_entry_obj.set_log_date(time_entry['log_date'])
        time_entry_obj.set_begin_time(time_entry['begin_time'])
        time_entry_obj.set_end_time(time_entry['end_time'])
        time_entry_obj.set_log_time(time_entry['log_time'])
        time_entry_obj.set_billed_status(time_entry['billed_status'])
        time_entry_obj.set_notes(time_entry['notes'])
        time_entry_obj.set_timer_started_at(time_entry['timer_started_at'])
        time_entry_obj.set_timer_duration_in_minutes(time_entry[\
        'timer_duration_in_minutes'])
        time_entry_obj.set_created_time(time_entry['created_time'])
        return time_entry_obj

    def get_comments(self, resp):
        """This method parses the given response and returns comments list 
            object.

        Args: 
            resp(dict): Response containing json object for comments list.

        Returns:
            list of instance: Comments list object.

        """
        comments = CommentList()
        for value in resp['comments']:
            comment = Comment()
            comment.set_comment_id(value['comment_id'])
            comment.set_project_id(value['project_id'])
            comment.set_description(value['description'])
            comment.set_commented_by_id(value['commented_by_id'])
            comment.set_commented_by(value['commented_by'])
            comment.set_is_current_user(value['is_current_user'])
            comment.set_date(value['date'])
            comment.set_date_description(value['date_description'])
            comment.set_time(value['time'])
            comments.set_comments(comment)
        return comments

    def get_comment(self, resp):
        """This method parses the given response and returns comment object.

        Args:
            resp(dict): Response containing json object for comment object.

        Returns: 
            instance: Comment object.

        """
        comment = resp['comment'] 
        comment_obj = Comment()
        comment_obj.set_comment_id(comment['comment_id'])
        comment_obj.set_project_id(comment['project_id'])
        comment_obj.set_description(comment['description'])
        comment_obj.set_commented_by_id(comment['commented_by_id'])
        comment_obj.set_commented_by(comment['commented_by'])
        comment_obj.set_date(comment['date'])
        comment_obj.set_date_description(comment['date_description'])
        comment_obj.set_time(comment['time'])
        return comment_obj

    def get_invoice_list(self, resp):
        """This method parses the given response and returns invoice list 
            object.

        Args:
            resp(dict): Response containing json object for invoice list 
                object.

        Returns:
            instance: Invoice list object.

        """
        invoices = InvoiceList()
        for value in resp['invoices']:
            invoice = Invoice()
            invoice.set_invoice_id(value['invoice_id'])
            invoice.set_customer_name(value['customer_name'])
            invoice.set_customer_id(value['customer_id'])
            invoice.set_status(value['status'])
            invoice.set_invoice_number(value['invoice_number'])
            invoice.set_reference_number(value['reference_number'])
            invoice.set_date(value['date'])
            invoice.set_due_date(value['due_date'])
            invoice.set_total(value['total'])
            invoice.set_balance(value['balance'])
            invoice.set_created_time(value['created_time'])
            invoices.set_invoices(invoice)
        page_context = resp['page_context']
        page_context_obj = PageContext()
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        invoices.set_page_context(page_context)
        return invoices

       
    



#$Id$

from books.model.PageContext import PageContext

class TaxList:
    """This class is used to create object for tax list."""
    def __init__(self):
        """Initialize parameters for tax list."""
        self.taxes = []
        self.page_context = PageContext()

    def set_taxes(self, tax):
        """Set taxes.
 
        Args:
            tax(instance): Tax object.

        """
        self.taxes.append(tax)

    def get_taxes(self):
        """Get taxes.

        Returns:
            list of instance: List of Tax object.

        """
        return self.taxes

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.
 
        """
        return self.page_context



#$Id$

class ExchangeRate:
    """This class is used to create object for Exchange rate."""
    def __init__(self):
        """Initialize parameters for exchange rate."""
        self.exchange_rate_id = ''
        self.currency_id = ''
        self.currency_code = ''
        self.effective_date = ''
        self.rate = 0.0

    def set_exchange_rate_id(self, exchange_rate_id):
        """Set exchange rate id.

        Args:
            exchange_rate_id(str): Exchange rate id.

        """
        self.exchange_rate_id = exchange_rate_id

    def get_exchange_rate_id(self):
        """Get exchange rate id.
 
        Returns:
            str: Exchange rate id.

        """
        return self.exchange_rate_id

    def set_currency_id(self, currency_id):
        """Set currency id.
       
        Args:
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self):
        """Get currency id.
 
        Returns:
            str: Currency id.

        """
        return self.currency_id

    def set_currency_code(self, currency_code):
        """Set currency code.

        Args:
            currency_code(str): Currency code.
 
        """
        self.currency_code = currency_code

    def get_currency_code(self):
        """Get currency code.

        Returns:
            str: Currency code.

        """
        return self.currency_code

    def set_effective_date(self, effective_date):
        """Set effective date.
 
        Args:
            effective_date(str): Effective date.

        """
        self.effective_date = effective_date

    def get_effective_date(self):
        """Get effective date.

        Returns:
            str: Effective date.

        """
        return self.effective_date

    def set_rate(self, rate):
        """Set rate.

        Args:
            rate(float): Rate.

        """
        self.rate = rate

    def get_rate(self):
        """Get rate.

        Returns:
            float: Rate.

        """
        return self.rate

    def to_json(self):
        """This method is used to create json object for exchange rate.

        Returns:
            dict: Dictionary containing json object for exchange rate.

        """
        data = {}
        if self.currency_id != '':
            data['currency_id'] = self.currency_id
        if self.currency_code != '':
            data['currency_code'] = self.currency_code
        if self.effective_date != '':
            data['effective_date'] = self.effective_date
        if self.rate > 0:
            data['rate'] = self.rate
        if self.effective_date != '':
            data['effective_date'] = self.effective_date
        return data



#$Id$

from books.model.PageContext import PageContext

class UserList: 
    """This class is used to create object for Users list."""
    def __init__(self):
        """Initialize parameters for users."""
        self.users = []
        self.page_context = PageContext()

    def set_users(self, user):
        """Set users.

        Args:
            user(instance): User object.

        """
        self.users.append(user)

    def get_users(self):
        """Get users.

        Returns:
            list of instance: List of user objects.

        """
        return self.users

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context




#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.SettingsParser import SettingsParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'items/'
zoho_http_client = ZohoHttpClient()
parser = SettingsParser()

class ItemsApi:
    """Items Api class is used to 

    1.List items.
    2.Get an item.
    3.Create an item.
    4.Update an item.
    5.Delete an item.
    6.Mark item as active.
    7.Mark item as inactive.
     
    """
    def __init__(self, authtoken, organization_id):
        """Initialize Settings Api using authtoken and organization id.

        Args:
            authtoken(str): User's Authtoken.
            organization_id(str): User's Organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id 
            }
    def list_items(self): 
        """Get the list of all active items with pagination.

        Returns: 
            instance: Items list object.

        """
        resp = zoho_http_client.get(base_url, self.details)
        return parser.get_items(resp) 

    def get(self, item_id):
        """Get details of an item.

        Args:
            item_id(str): Item id.

        Returns:
            instance: Item object.

        """
        url = base_url + item_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_item(resp) 
    
    def create(self, item):
        """Create a new item.

        Args:
            item(instance): Item object.

        Returns:
            instance: Item object.

        """
        json_obj = dumps(item.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.post(base_url, self.details, data)
        return parser.get_item(resp) 

    def update(self, item_id, item):
        """Update an existing item.

        Args:
            item_id(str): Item id.
            item(instance): Item object.

        Returns:
            instance: Item object.
 
        """
        url = base_url + item_id 
        json_obj = dumps(item.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.put(url, self.details, data) 
        return parser.get_item(resp) 

    def delete_item(self, item_id):
        """Delete an item.

        Args:
            item_id(str): Item id.

        Returns:
            str: Success message('The item has been deleted.').

        """ 
        url = base_url + item_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
    
    def mark_item_as_active(self, item_id):
        """Mark an item as active.

        Args:
            item_id(str): Item id.
 
        Returns:
            str: Success message('The item has been marked as active.').
   
        """
        url = base_url + item_id + '/active'
        data = { 
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp) 

    def mark_item_as_inactive(self, item_id):
        """Mark an item as inactive.

        Args:
            item_id(str): Item id.

        Returns:
            str: Success message('The item has been marked as inactive.').

        """
        url = base_url + item_id + '/inactive'
        data = { 
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp) 



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.SettingsParser import SettingsParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'organizations/'
zoho_http_client = ZohoHttpClient()
parser = SettingsParser()

class OrganizationsApi:
    """Organization api is used to

    1.List organizations.
    2.Get the details of an organization.
    3.Create an organization.
    4.Update an organization.
    
    """
    def __init__(self, authtoken, organization_id):
        """Initialize Settings Api using authtoken and organization id.

        Args:
            authtoken(str): User's Authtoken.
            organization_id(str): User's Organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id 
            }

    def get_organizations(self):
        """Get list of all organizations.

        Returns:
            instance: Organizations list object.

        """
        resp = zoho_http_client.get(base_url, self.details) 
        return parser.get_organizations(resp) 
 
    def get(self, organization_id):
        """Get organization id.

        Args:
            organization_id(str): Organization id.

        Returns:
            instance: Organization object.

        """
        url = base_url + organization_id
        resp = zoho_http_client.get(url, self.details) 
        return parser.get_organization(resp) 

    def create(self, organization):
        """Create an organization.

        Args:
            organization(instance): Organization object.

        Returns:
            instance: Organization object.

        """
        json_obj = dumps(organization.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.post(base_url, self.details, data)
        return parser.get_organization(resp)

    def update(self, organization_id, organization):
        """Update organization.

        Args:
            organization_id(str): ORganization id.
            organization(instance): Organization object.

        Returns:
            instance: Organization object.

        """
        url = base_url +  organization_id
        json_obj = dumps(organization.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_organization(resp) 



#$Id$#

from books.model.BaseCurrencyAdjustment import BaseCurrencyAdjustment
from books.model.BaseCurrencyAdjustmentList import BaseCurrencyAdjustmentList
from books.model.PageContext import PageContext
from books.model.Account import Account

class BaseCurrencyAdjustmentParser: 
    """This class is used to parse the json object for Base Currency 
        adjustment Api."""

    def get_list(self, resp):
        """This method parses the given response and returns base currency 
            adjustment list object.

        Args:
            resp(dict): Response containing json for base currency adjustments 
                list.

        Returns:
            instance: Base currency list object.

        """
        base_currency_adjustment_list = BaseCurrencyAdjustmentList()
        for value in resp['base_currency_adjustments']:
            base_currency_adjustment = BaseCurrencyAdjustment()
            base_currency_adjustment.set_base_currency_adjustment_id(\
            value['base_currency_adjustment_id'])
            base_currency_adjustment.set_adjustment_date(value[\
            'adjustment_date'])
            base_currency_adjustment.set_exchange_rate(value['exchange_rate'])
            base_currency_adjustment.set_currency_id(value['currency_id'])
            base_currency_adjustment.set_currency_code(value['currency_code'])
            base_currency_adjustment.set_notes(value['notes'])
            base_currency_adjustment.set_gain_or_loss(value['gain_or_loss'])
            base_currency_adjustment_list.set_base_currency_adjustments(\
            base_currency_adjustment)
        page_context = resp['page_context']
        page_context_obj = PageContext()
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        base_currency_adjustment_list.set_page_context(page_context_obj)
        return base_currency_adjustment_list

    def get_base_currency_adjustment(self, resp):
        """This method parses the given response and returns the base currency 
            adjustment details object.

        Args:
            resp(dict): Response containing json object for base currency 
                adjustment.

        Returns:
            instance: Base currency adjustment object.

        """
        data = resp['data']
        base_currency_adjustment = BaseCurrencyAdjustment()
        base_currency_adjustment.set_base_currency_adjustment_id(data[\
        'base_currency_adjustment_id']) 
        base_currency_adjustment.set_currency_id(data['currency_id'])
        base_currency_adjustment.set_currency_code(data['currency_code'])
        base_currency_adjustment.set_adjustment_date(data['adjustment_date'])
        base_currency_adjustment.set_exchange_rate(data['exchange_rate'])
        base_currency_adjustment.set_notes(data['notes'])
        for value in data['accounts']:
            account = Account()
            account.set_account_id(value['account_id'])
            account.set_account_name(value['account_name'])
            account.set_bcy_balance(value['bcy_balance'])
            account.set_fcy_balance(value['fcy_balance'])
            account.set_adjusted_balance(value['adjusted_balance'])
            account.set_gain_or_loss(value['gain_or_loss'])
            account.set_gl_specific_type(value['gl_specific_type'])
            base_currency_adjustment.set_accounts(account)
        return base_currency_adjustment

    def list_account_details(self, resp):
        """This method parses the given response and returns list of account 
            details for base currency adjustment.

        Args:
            resp(dict): Response containing json object for account details list.

        Returns:
            instance: Base currency adjustment object.

        """
        data = resp['data']
        base_currency_adjustment = BaseCurrencyAdjustment()
        base_currency_adjustment.set_adjustment_date(data['adjustment_date'])
        base_currency_adjustment.set_adjustment_date_formatted(data[\
        'adjustment_date_formatted'])
        base_currency_adjustment.set_exchange_rate(data['exchange_rate'])
        base_currency_adjustment.set_exchange_rate_formatted(data[\
        'exchange_rate_formatted'])
        base_currency_adjustment.set_currency_id(data['currency_id'])
        for value in data['accounts']:
            accounts = Account()
            accounts.set_account_id(value['account_id'])
            accounts.set_account_name(value['account_name'])
            accounts.set_gl_specific_type(value['gl_specific_type'])
            accounts.set_fcy_balance(value['fcy_balance'])
            accounts.set_fcy_balance_formatted(value['fcy_balance_formatted'])
            accounts.set_bcy_balance(value['bcy_balance'])
            accounts.set_bcy_balance_formatted(value['bcy_balance_formatted'])
            accounts.set_adjusted_balance(value['adjusted_balance'])
            accounts.set_adjusted_balance_formatted(value[\
            'adjusted_balance_formatted'])
            accounts.set_gain_or_loss(value['gain_or_loss'])
            accounts.set_gain_or_loss_formatted(value['gain_or_loss_formatted'])
            base_currency_adjustment.set_accounts(accounts)
        base_currency_adjustment.set_notes(data['notes'])
        base_currency_adjustment.set_currency_code(data['currency_code'])
        return base_currency_adjustment

    def get_message(self, resp): 
        """This method parses the given response and returns message.

        Args:
            resp(dict): Response containing json object for message.

        Returns:
            str: Success message.

        """
        return resp['message']



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.RecurringExpensesParser import RecurringExpensesParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'recurringexpenses/'
parser = RecurringExpensesParser()
zoho_http_client = ZohoHttpClient()

class RecurringExpensesApi:
    """Recurring Expenses Api is used to 

    1.List recurring expenses with pagination .
    2.Get the details of a recurring expense.
    3.Create a recurring expense.
    4.Update an existing recurring expense.
    5.Delete an existing recurring expense.
    6.Stop an active recurring expense.
    7.Resume a stopped recurring expense.
    8.List child expenses created from recurring expense.
    9.Get history and comments of a recurring expense.

    """
    def __init__(self, authtoken, organization_id):
        """Initialize Recurring Expenses Api using user's authtoken and 
            organization id. 
     
        Args:
            authotoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id
            } 
  
    def get_recurring_expenses(self, param=None):
        """List recurring expenses with pagination.

        Args:
            parameter(dict, optional): Filter with which the list has to be 
                displayed.

        Returns:
            instance: Recurring expenses list object.

        """
        resp = zoho_http_client.get(base_url, self.details, param) 
        return parser.get_list(resp) 

    def get(self, recurring_expense_id):
        """Get details of a recurring expense.
   
        Args:
            recurring_expense_id: Recurring expense id.

        Returns:
            instance: Recurring expense object.

        """
        url = base_url + recurring_expense_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_recurring_expense(resp) 

    def create(self, recurring_expenses):
        """Create a recurring expense.

        Args:
            recurring_expenses(instance): Recurring expense object.

        Returns:
            instance: Recurring expense object.
 
        """
        json_object = dumps(recurring_expenses.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(base_url, self.details, data)
        return parser.get_recurring_expense(resp) 

    def update(self, recurring_expense_id, recurring_expenses):
        """Update an existing recurring expense.

        Args:
            recurring_expense_id(str): Recurring expense id.
            recurring_expenses(instance): Recurring expense object.

        Returns:
            instance: Recurring expense object.

        """ 
        url = base_url + recurring_expense_id
        json_object = dumps(recurring_expenses.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_recurring_expense(resp)
  
    def delete(self, recurring_expense_id):
        """Delete an existing recurring expense.
 
        Args:
            recurring_expense_id(str): Recurring expense id.

        Returns:
            str: Success message('The recurring expense has been deleted.').

        """
        url = base_url + recurring_expense_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
 
    def stop_recurring_expense(self, recurring_expense_id):
        """Stop an active recurring expense.

        Args:
            recurring_expense_id(str): Recurring expense id.

        Returns:
            str: Success message('The recurring expense has been stopped.').

        """
        url = base_url + recurring_expense_id + '/status/stop'
        resp = zoho_http_client.post(url, self.details, '')
        return parser.get_message(resp) 
  
    def resume_recurring_expense(self, recurring_expense_id):
        """Resume a stopped recurring expense.

        Args:
            recurring_expense_id(str): Recurring expense id.

        Returns:
            str: Success message('Resume a stopped recurring expense.').

        """
        url = base_url + recurring_expense_id + '/status/resume'
        resp = zoho_http_client.post(url, self.details, '')
        return parser.get_message(resp) 

    def list_child_expenses_created(self, recurring_expense_id, parameter=None):
        """List child expenses created.

        Args:
            recurring_expense_id(str): Recurring Expense id.
            sort_column(str, optional): Sort child expenses created. Allowed 
                values are date, account_name, vendor_name, 
                paid_through_account_name, customer_name and total.

        Returns:
            instance: Expense history object.
            
        """
        url = base_url + recurring_expense_id + '/expenses'
        if parameter is not None:
            query = {
                'sort_column': parameter
                }
        else:
            query = None 
        resp = zoho_http_client.get(url, self.details, query)
        return parser.get_expense_history(resp) 
  
    def list_recurring_expense_comments_and_history(self, recurring_expense_id):
        """Get history and comments of recurring expense.

        Args: 
            recurring_expense_id(str): Recurring expense id.

        Returns:
            instance: Comments list object.

        """
        url = base_url + recurring_expense_id + '/comments'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_comments(resp) 

  



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.VendorPaymentsParser import VendorPaymentsParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'vendorpayments/'
parser = VendorPaymentsParser()
zoho_http_client = ZohoHttpClient()

class VendorPaymentsApi:
    """Vendor Payaments Api is used to

    1.List all the paymnets made to your vendor.
    2.Get the details of a vendor payment.
    3.Create a payment made to the vendor.
    4.Update an existing vendor payment.
    5.Delete an existing vendor payment.

    """
    def __init__(self, authtoken, organization_id):
        """Initialize Vendor payments api using user's authtoken and 
            organization id.

        Args: 
            authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id
            }
   
    def get_vendor_payments(self, parameter=None):
        """List all the payments made to the vendor.

        Args:
            parameter(dict, optional): Filter with which the list has to be 
                displayed.

        Returns:
            instance: Vendor payments list object.

        """
        resp = zoho_http_client.get(base_url, self.details, parameter)
        return parser.get_list(resp) 

    def get(self, payment_id):
        """Get the details of vendor payment.
 
        Args:
            payment_id(str): Payment id.
 
        Returns:
            instance: Vendor payments object.

        """
        url = base_url + payment_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_vendor_payment(resp) 

    def create(self, vendor_payments):
        """Create a payment made to vendor.

        Args:
            vendor_payments(instance): Vendor payments object.

        Returns:
            instance: Vendor payments object.

        """
        json_object = dumps(vendor_payments.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(base_url, self.details, data)
        return parser.get_vendor_payment(resp) 

    def update(self, payment_id, vendor_payments):
        """Update an existing vendor payment.
      
        Args:
            payment_id(str): Payment id.
            vendor_payments(instance): Vendor payments object.

        Returns:
            instance: Vendor payments object.
 
        """
        url = base_url + payment_id
        json_object = dumps(vendor_payments.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_vendor_payment(resp) 

    def delete(self, payment_id):
        """Delete an existing vendor payment.
 
        Args:
            payment_id(str): Payment id.

        Returns:
            str: Success message('The paymenet has been deleted.').

        """
        url = base_url + payment_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 







#$Id$

class CreditNoteRefundList:
    """This class is used to create object for Creditnotes Refund."""
    def __init__(self):
        """Initialize parameters for creditnotes refund list."""
        self.creditnote_refunds=[]
        self.page_context={}
   
    def set_creditnote_refunds(self, creditnote_refunds):
        """Set creditnote refunds.
   
        Args:
            creditnote_refunds(instance): Creditnote refunds object.
 
        """
        self.creditnote_refunds.append(creditnote_refunds)
 
    def get_creditnote_refunds(self):
        """Get creditnote refunds
        
        Returns:
            list: List of creditnote refunds.

        """
        return self.creditnote_refunds

    def set_page_context(self, page_context):
        """Set page context.
        
        Returns:
            page_context(instance): Page context object.
 
        """
        self.page_context=page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context
